import { Component, OnDestroy, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { NavController } from '@ionic/angular';
import { AuthActions, AuthService, IAuthAction } from 'ionic-appauth';
import { Subscription } from 'rxjs';
import { IdpInformationService } from 'src/app/services';
import { AuthenticationService } from 'src/app/services/authentication/authentication.service';
import { PAGES } from 'src/app/utility';

@Component({
  template: '<p>Caricamento...</p>'
})
export class AuthCallbackPage implements OnInit, OnDestroy {
  sub: Subscription;
  user$ = this.auth.user$;

  constructor(
    private auth: AuthService,
    private navCtrl: NavController,
    private router: Router,
    private authService: AuthenticationService,
    private idpInformationService: IdpInformationService) {
  }

  ngOnInit() {
    this.sub = this.auth.events$.subscribe((action) => this.postCallback(action));
    this.auth.authorizationCallback(window.location.origin + this.router.url);
  }

  ngOnDestroy() {
    this.sub.unsubscribe();
  }

  postCallback(action: IAuthAction) {
    if (action.action === AuthActions.SignInSuccess) {
      this.authService.login();
      setTimeout(() => {
        this.idpInformationService.saveUserInformation();
      });
      this.navCtrl.navigateRoot(PAGES.home_page);
    }

    if (action.action === AuthActions.SignInFailed) {
      this.authService.logout();
      this.navCtrl.navigateRoot(PAGES.landing_page);
    }
  }

  public getUserInfo(): Promise<void> {
    return this.auth.loadUserInfo();
  }


}
