import { Component, OnInit } from '@angular/core';
import { GlobalService, StorageService, UtenzaService } from 'src/app/services';
import { DURATIONTOAST, IMAGE } from 'src/app/utility/global-constant';

import { TipologiaNotifica, UpdateTipologieNotificheRequest } from '../../../models';

@Component({
	selector: 'app-gestione-notifiche',
	templateUrl: './gestione-notifiche.page.html',
	styleUrls: ['./gestione-notifiche.page.scss'],
})
export class GestioneNotifichePage implements OnInit {
	titleHeader: string = 'Gestione Notifiche';
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	lista_TN_start: Array<TipologiaNotifica>;
	lista_TN_temp: Array<TipologiaNotifica>;
	lista_TN_request: Array<TipologiaNotifica>;
	updateTipologieNotificheRequest: UpdateTipologieNotificheRequest;
	tipologia_temp: TipologiaNotifica;
	showCardDSA: boolean;
	showCardLTP: boolean;
	showButtonUpdate: boolean;
	zeroTipologieNotifiche: boolean;
	sizeSkelNotif: number;
	allOrNotSelected: string;
	selectAll: string = 'allSelect';
	deselectAll: string = 'allDeselect';
	isAtomic: boolean;

	constructor(private storageService: StorageService, private utenzaService: UtenzaService, private globalService: GlobalService) {
		this.setUtility();
		this.setVariabiliAllFalse();
		this.zeroTipologieNotifiche = false;
		this.getNotifichePerUtente();
	}

	ngOnInit() {}

	setUtility() {
		this.sizeSkelNotif = 4;
	}

	setVariabiliAllFalse() {
		this.showCardDSA = false;
		this.showCardLTP = false;
		this.showButtonUpdate = false;
	}

	setVariabiliAllTrue() {
		this.showCardDSA = true;
		this.showCardLTP = true;
		this.showButtonUpdate = true;
	}

	setListaTN() {
		//TODO settare due liste
		this.lista_TN_temp = [
			{ codice: 'MACS_PRE', titolo: 'Prenotazione', stato: 'Y' },
			{ codice: 'MACS_COM', titolo: 'Comunicazione', stato: 'N' },
			{ codice: 'MACS_AVVISO', titolo: 'Avvertenza', stato: 'Y' },
			{ codice: 'MACS_POL', titolo: 'Politica locale', stato: 'N' },
		];
		this.lista_TN_start = [
			{ codice: 'MACS_PRE', titolo: 'Prenotazione', stato: 'Y' },
			{ codice: 'MACS_COM', titolo: 'Comunicazione', stato: 'N' },
			{ codice: 'MACS_AVVISO', titolo: 'Avvertenza', stato: 'Y' },
			{ codice: 'MACS_POL', titolo: 'Politica locale', stato: 'N' },
		];
	}

	cambiaStato(index: number) {
		if (!this.isAtomic) {
			if (this.lista_TN_temp[index].stateBoolean) {
				this.lista_TN_temp[index].stato = 'N';
			} else {
				this.lista_TN_temp[index].stato = 'Y';
			}
		}
		return false;
	}

	changeAllOrNotSelected() {
		this.isAtomic = true;
		switch (this.allOrNotSelected) {
			case this.selectAll:
				this.setAllStatesOfNotifiche(true);
				break;
			case this.deselectAll:
				this.setAllStatesOfNotifiche(false);
				break;
			default:
				break;
		}
	}

	setAllStatesOfNotifiche(stato: boolean) {
		if (this.lista_TN_temp && this.lista_TN_temp.length > 0) {
			for (let i = 0; i < this.lista_TN_temp.length; i++) {
				if (stato) {
					this.lista_TN_temp[i].stateBoolean = true;
					this.lista_TN_temp[i].stato = 'Y';
				} else {
					this.lista_TN_temp[i].stateBoolean = false;
					this.lista_TN_temp[i].stato = 'N';
				}
			}
		}
		setTimeout(() => {
			this.isAtomic = false;
		}, 500);
	}

	checkChangeStateOfItem() {
		this.lista_TN_request = [];
		for (let i = 0; i < this.lista_TN_start.length; i++) {
			if (this.lista_TN_start[i].stato !== this.lista_TN_temp[i].stato) {
				this.lista_TN_request.push(this.lista_TN_temp[i]);
			}
		}
	}

	getNotifichePerUtente() {
		this.storageService.getUserInfo().then((userInfo) => {
			if (userInfo) {
				this.utenzaService.getNotifichePerUtente(userInfo.identifier).subscribe(
					(result) => {
						if (result.success) {
							if (result.data) {
								this.lista_TN_start = result.data;
								if (this.lista_TN_start && this.lista_TN_start.length > 0) {
									this.setVariabiliAllTrue();
									this.zeroTipologieNotifiche = false;
									this.setListaTipNotificheTemp();
								} else {
									this.setVariabiliAllFalse();
									this.zeroTipologieNotifiche = true;
								}
							}
						}
					},
					(error) => {
						this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
						this.setVariabiliAllFalse();
						this.zeroTipologieNotifiche = true;
					}
				);
			}
		});
	}

	updateNotifichePerUtente() {
		this.checkChangeStateOfItem();
		this.storageService.getUserInfo().then((userInfo) => {
			if (userInfo) {
				this.updateTipologieNotificheRequest = {
					spid_code: userInfo.identifier,
					listaNotifiche: this.lista_TN_request,
				};
				this.utenzaService.updateNotifichePerUtente(this.updateTipologieNotificheRequest).subscribe(
					(result) => {
						if (result.success) {
							this.globalService.presentToast('Aggiornamento effettuato con successo', DURATIONTOAST.default);
							this.getNotifichePerUtente();
						}
					},
					(error) => {
						this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
					}
				);
			}
		});
	}

	setListaTipNotificheTemp() {
		this.lista_TN_temp = [];
		for (let tipologia of this.lista_TN_start) {
			this.tipologia_temp = {
				codice: tipologia.codice,
				titolo: tipologia.titolo,
				stato: tipologia.stato,
				stateBoolean: tipologia.stato === 'Y' ? true : false,
			};
			this.lista_TN_temp.push(this.tipologia_temp);
		}
	}
}
