import { DatePipe, Location } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { AlertController } from '@ionic/angular';
import { CalendarMode, QueryMode } from 'ionic2-calendar/calendar';
import * as moment from 'moment';
import { pluck } from 'rxjs/operators';
import { PStep2Step3, PStep3Step4 } from 'src/app/models';
import { DURATIONTOAST, IMAGE, PAGES } from 'src/app/utility/global-constant';

import {
    Anagrafica,
    AnnullaPrenotazioneRequest,
    AuditRequest,
    AuditSlot,
    ConfermaPrenotazioneRequest,
    ConfermaPrenotazioneResponse,
    ConfermaPrenotazioneTotaleRequest,
    CoreRequest,
    Evento,
    InfoRisorsa,
    InitPrenotazioneRequest,
    InitPrenotazioneResponse,
    PeriodiDisponibiliRequest,
    PeriodiDisponibiliResponse,
    Risorsa,
    Risposta,
    ValoreRisposta,
} from '../../../models';
import { Slot } from '../../../models/slot';
import { GlobalService, PrenotazioneService, StorageService } from '../../../services';

@Component({
    selector: 'app-prenotazione-step3',
    templateUrl: './prenotazione-step3.page.html',
    styleUrls: ['./prenotazione-step3.page.scss'],
})
export class PrenotazioneStep3Page implements OnInit {
    titleHeader: string = PAGES.prenotazioneStep3.title;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    infoRisorsa: InfoRisorsa;
    minutiPrenotabili: number;
    nomeUfficio: string;
    idUfficio: number;
    startHour: number = 9;
    endHour: number = 18;
    allDayLabel: string = '';
    startingDayMonth: number = 1;
    startingDayWeek: number = 1;
    formatHourColumn: string = 'HH:mm';
    timeInterval: string = '30'; // TODO impostarlo a minutiPrenotabili
    queryMode: QueryMode = 'remote';
    events = [];
    eventoSelezionato: Evento;
    eventSource;
    viewTitle;
    isToday: boolean;
    valoriRisposta: Array<ValoreRisposta>;
    anagraficaFruitore: Anagrafica;
    anagraficaPresentante: Anagrafica;
    coreRequest: CoreRequest;
    uffOrRis: Risorsa;
    data_inzio_query: string;
    data_fine_query: string;
    pdRequest: PeriodiDisponibiliRequest;
    pdResponse: PeriodiDisponibiliResponse;
    eventoTemp: Evento;
    slotsTemp: Array<Slot>;
    ipRequest: InitPrenotazioneRequest;
    ipResponse: InitPrenotazioneResponse;
    dataMaxPrenot: Date;
    dataMinPrenot: Date;
    list_mesi: Array<String>;
    dataPrenotazione: string;
    list_risposte: Array<Risposta>;
    rispostaTemp: Risposta;
    cpRequest: ConfermaPrenotazioneRequest;
    cptRequest: ConfermaPrenotazioneTotaleRequest;
    cpResponse: ConfermaPrenotazioneResponse;
    apRequest: AnnullaPrenotazioneRequest;
    lista_slot_occupati: Array<AuditSlot>;
    slotOccupatoTemp: AuditSlot;
    auditRequest: AuditRequest;
    questionarioNotObbligato: string;
    showLegenda: boolean;
    lockSwipeToPrev: boolean;
    canBack: boolean;
    loadPeriodiDisponibili: boolean = false;
    calendar = {
        mode: 'week' as CalendarMode, // inizio
        locale: 'it-IT',
        currentDate: new Date(),
        dateFormatter: {
            formatMonthViewDay: function (date: Date) {
                return date.getDate().toString();
            },
            formatMonthViewDayHeader: function (date: Date) {
                return 'MonMH';
            },
            formatMonthViewTitle: function (date: Date) {
                return 'testMT';
            },
            formatWeekViewDayHeader: function (date: Date) {
                return 'MonWH';
            },
            formatWeekViewTitle: function (date: Date) {
                return 'testWT';
            },
            formatWeekViewHourColumn: function (date: Date) {
                return 'testWH';
            },
            formatDayViewHourColumn: function (date: Date) {
                return 'testDH';
            },
            formatDayViewTitle: function (date: Date) {
                return 'testDT';
            },
        },
    };

    constructor(
        private globalService: GlobalService,
        private router: Router,
        private activatedRoute: ActivatedRoute,
        public location: Location,
        private datePipe: DatePipe,
        private storageService: StorageService,
        private alertController: AlertController,
        private prenotazioneService: PrenotazioneService
    ) {
        this.getParams();
    }

    ngOnInit() {
        /* this.coreRequest = {
      cfFruitore: 'VZURMO31L50L377E',
      cfUser: 'QBDDMT28D42L139O'
    } */
        /*     this.coreRequest = {
      cfFruitore: "SLTMRA80A01A944H",
      cfUser: "SLTMRA80A01A944H",
    }; */
        //setting della data inizio e fine allo start
        // this.data_inzio_query = this.datePipe.transform(new Date(), "dd/MM/yyyy");
        // this.data_fine_query = "24/07/2022";
        // console.log("calendar: ", this.calendar);
        // /* this.data_fine_query = this.datePipe.transform(
        //    evento.endTime,
        //    "dd/MM/yyyy"
        //  ); */
        // setTimeout(() => {
        //   this.getPeriodiDisponibili();
        // }, 3000);
    }

    getParams() {
        this.activatedRoute.params.pipe(pluck('params')).subscribe((result) => {
            let pStep2Step3: PStep2Step3;
            pStep2Step3 = this.globalService.decodeParams(result);
            /*  this.anagraficaFruitore = pSte1Step2.anagraficaFruitore;
           this.anagraficaPresentante = pSte1Step2.anagraficaPresentante; */
            this.setVarFromParams(pStep2Step3);
            this.setUtility();
            this.setLista();
            this.getValoriRisposta();
        });
    }

    setVarFromParams(pStep2Step3: PStep2Step3) {
        this.anagraficaFruitore = pStep2Step3.anagraficaFruitore;
        this.anagraficaPresentante = pStep2Step3.anagraficaPresentante;
        this.uffOrRis = pStep2Step3.uffOrRis;
        this.infoRisorsa = pStep2Step3.infoRisorsa;
        this.nomeUfficio = pStep2Step3.nomeUfficio;
        this.idUfficio = pStep2Step3.idUfficio;
        this.questionarioNotObbligato = pStep2Step3.questionarioNotObbligato;
        // TODO impostarlo quando i dati sono certi
        this.minutiPrenotabili = pStep2Step3.minutiPrenotabili;
        //this.minutiPrenotabili = 30;
        this.valoriRisposta = pStep2Step3.valoriRisposta;
        this.infoRisorsa.giorniMinPrenot = null;
        this.infoRisorsa.giorniMaxPrenot = null;

        //FORMAT DATA NASCITA
        this.anagraficaFruitore.dataNascita = this.datePipe.transform(this.anagraficaFruitore.dataNascita, 'yyyy-MM-dd');
        this.anagraficaPresentante.dataNascita = this.datePipe.transform(this.anagraficaPresentante.dataNascita, 'yyyy-MM-dd');
    }

    setUtility() {
        this.coreRequest = {
            cfFruitore: this.anagraficaFruitore?.cf,
            cfUser: this.anagraficaPresentante?.cf,
        };
        this.showLegenda = true;
        this.lockSwipeToPrev = true;
        this.setEventoEmpty();
        this.setMinMaxPrenotabili();
    }

    setLista() {
        this.list_risposte = [];
        this.lista_slot_occupati = [];
        this.list_mesi = ['Gennnaio', 'Febbraio', 'Marzo', 'Aprile', 'Maggio', 'Giugno', 'Luglio', 'Agosto', 'Settembre', 'Ottobre', 'Novembre', 'Dicembre'];
    }

    ionViewDidLoad() {
        console.log('ionViewDidLoad PrenotazioneStep3Page');
    }

    onViewTitleChanged(title) {
        if (this.controllaTitolo(title)) {
            this.viewTitle = this.cambiaTitolo(title);
        } else {
            this.viewTitle = title;
        }
    }

    onEventSelected(event) {
        let resultControllo = this.controllaMinMaxGiorniPrenotabili(event);
        if (!event.occupato && resultControllo) {
            if (!(this.eventoSelezionato.uuid !== '')) {
                this.eventoSelezionato = event;
                /* console.log(this.eventoSelezionato); */
                this.initPrenotazione();
            } else if (this.eventoSelezionato.uuid === event.uuid) {
                this.confermaOrAnnulla();
            } else {
                this.globalService.presentToast('Conferma la prenotazione per proseguire', DURATIONTOAST.default);
                return;
            }
        } else {
            if (event.occupato) {
                if (this.eventoSelezionato.uuid !== '') {
                    this.globalService.presentToast('Conferma la prenotazione per proseguire', DURATIONTOAST.default);
                } else {
                    this.addSlotOccuppato(event);
                    this.globalService.presentToast('La risorsa è occupata', DURATIONTOAST.default);
                }
            } else {
                this.globalService.presentToast('Non è prenotabile', DURATIONTOAST.default);
            }
        }
    }

    // CAMBIA LA VISUALIZZAZIONE
    changeMode(mode) {
        if (mode === 'month') {
            this.showLegenda = false;
        } else {
            this.showLegenda = true;
        }
        this.calendar.mode = mode;
    }

    today() {
        this.calendar.currentDate = new Date();
    }

    onCurrentDateChanged(event: Date) {
        console.log(event)
        let today = new Date();
        today.setHours(0, 0, 0, 0);
        event.setHours(0, 0, 0, 0);
        this.isToday = today.getTime() === event.getTime();
        if (this.calendar.mode === 'month') {
            if (event.getFullYear() < today.getFullYear() || (event.getFullYear() === today.getFullYear() && event.getMonth() <= today.getMonth())) {
                this.lockSwipeToPrev = true;
            } else {
                this.lockSwipeToPrev = false;
            }
        }
    }

    onRangeChanged(evento) {
        const today = new Date().getTime();
        const start = new Date(evento.startTime).getTime();
        const end = new Date(evento.endTime).getTime();

        if (start < today && today < end) {
            this.lockSwipeToPrev = true;
        } else {
            this.lockSwipeToPrev = false;
        }
        if (start >= today || (start < today && today < end)) {
            this.data_inzio_query = this.datePipe.transform(evento.startTime, 'yyyy-MM-dd');
            this.data_fine_query = this.datePipe.transform(evento.endTime, 'yyyy-MM-dd');
            this.getPeriodiDisponibili();
        } else {
            return;
        }
    }

    markDisabled = (date: Date) => {
        let current = new Date();
        current.setHours(0, 0, 0);
        return date < current;
    };

    getPeriodiDisponibili() {
        /*     this.pdRequest = {
          cfFruitore: this.coreRequest.cfFruitore, cfUser: this.coreRequest.cfUser,
          idRisorsa: this.uffOrRis.id, da: this.data_inzio_query, a: this.data_fine_query
        }; */
        this.pdRequest = {
            cfFruitore: this.coreRequest.cfFruitore,
            cfUser: this.coreRequest.cfUser,
            idRisorsa: this.uffOrRis.id,
            da: this.data_inzio_query,
            a: this.data_fine_query,
        };
        this.loadPeriodiDisponibili = true;
        this.prenotazioneService.getPeriodiDisponibili(this.pdRequest).subscribe(
            (result) => {
                if (result.success) {
                    this.pdResponse = result.data.content;
                    if (this.pdResponse.slots) {
                        this.setPeriodiDisponibili();
                    }
                    if (this.pdResponse?.slots?.length === 0) {
                        this.globalService.presentToast('Non ci sono slot prenotabili', DURATIONTOAST.default);
                    }
                }
            },
            (error) => {
                this.loadPeriodiDisponibili = false;
            },
            () => {
                setTimeout(() => {
                    this.loadPeriodiDisponibili = false;
                }, 500);
            }
        );
    }

    setPeriodiDisponibili() {
        this.events = [];
        let current = new Date();
        if (this.pdResponse.slots) {
            this.slotsTemp = this.pdResponse.slots;
            for (let i = 0; i < this.slotsTemp.length; i++) {
                if (this.slotsTemp[i].uuid === this.eventoSelezionato.uuid) {
                    this.events.push(this.eventoSelezionato);
                } else {
                    let data_inizio = new Date(Number(this.slotsTemp[i].inizio));
                    let data_fine = new Date(Number(this.slotsTemp[i].fine));
                    if (data_inizio >= current) {
                        this.eventoTemp = {
                            title: '',
                            startTime: data_inizio,
                            endTime: data_fine,
                            inizio: this.slotsTemp[i].inizio,
                            fine: this.slotsTemp[i].fine,
                            occupato: this.slotsTemp[i].occupato,
                            uuid: this.slotsTemp[i].uuid,
                            prezzo: this.slotsTemp[i].prezzo,
                            selezionato: false,
                            allDay: false,
                            qtaDisponibile: this.slotsTemp[i].qtaDisponibile,
                        };
                        this.events.push(this.eventoTemp);
                    } else {
                        // return;
                    }
                }
            }
            this.eventSource = this.events;
        } else {
            this.eventSource = this.events;
        }
    }

    trasformStringtoDate(stringDate: string): Date {
        let day = stringDate.substr(0, 2);
        let month = stringDate.substr(3, 2);
        let year = stringDate.substr(6, 4);
        let hour = stringDate.substr(11, 2);
        let minutes = stringDate.substr(14, 2);
        let date: Date = new Date(Number(year), Number(month) - 1, Number(day), Number(hour), Number(minutes), 0, 0);
        return date;
    }

    setEventoEmpty() {
        this.eventoSelezionato = {
            title: '',
            startTime: null,
            endTime: null,
            occupato: false,
            uuid: '',
            prezzo: 0,
            selezionato: false,
            allDay: false,
            qtaDisponibile: 0,
            fine: '',
            inizio: '',
        };
    }

    setFlagSelezionato(uuid: string, selezionato: boolean) {
        for (let i = 0; i < this.events.length; i++) {
            if (this.events[i].uuid === uuid && selezionato) {
                this.events[i].selezionato = true;
            } else if (this.events[i].uuid === uuid && !selezionato) {
                this.events[i].selezionato = false;
            }
        }
        this.eventSource = this.events;
    }

    getColor(event): string {
        if (event.occupato) {
            return '#ff0000';
        } else if (event.selezionato) {
            return '#fdff09';
        } else {
            return '#00bb2d';
        }
    }

    initPrenotazione() {
        this.ipRequest = {
            cfUser: this.coreRequest.cfUser,
            cfFruitore: this.coreRequest.cfFruitore,
            idRisorsa: this.uffOrRis.id,
            openApi: this.infoRisorsa?.openApi,
            slots: [
                {
                    fine: this.eventoSelezionato.fine,
                    inizio: this.eventoSelezionato.inizio,
                    occupato: this.eventoSelezionato.occupato,
                    prezzo: this.eventoSelezionato.prezzo,
                    uuid: this.eventoSelezionato.uuid,
                    qtaDisponibile: this.eventoSelezionato.qtaDisponibile,
                },
            ],
        };
        this.prenotazioneService.initPrenotazione(this.ipRequest).subscribe(
            (result) => {
                if (result.success) {
                    this.ipResponse = result.data.content;
                    if (this.ipResponse) {
                        this.setFlagSelezionato(this.eventoSelezionato.uuid, true);
                        this.dataPrenotazione = this.transformDateCompletToString(this.eventoSelezionato.startTime);
                        this.confermaOrAnnulla();
                        //setTimeout(() => this.annullaPrenotazione(this.eventoSelezionato.uuid), ms);
                    }
                }
            },
            (error) => {
                this.globalService.presentToast('Il servizio non è al momento disponibile', DURATIONTOAST.default);
                this.setEventoEmpty();
                this.setFlagSelezionato(this.eventoSelezionato.uuid, false);
            }
        );
    }

    confermaPrenotazione() {
        this.storageService.getUserInfo();
        this.storageService.getUserInfo().then((userInfo) => {
            if (userInfo && userInfo?.identifier) {
                this.cpRequest = {
                    anagraficaFruitore: this.anagraficaFruitore,
                    anagraficaPresentante: this.anagraficaPresentante,
                    cfFruitore: this.anagraficaFruitore.cf,
                    cfUser: this.anagraficaPresentante.cf,
                    note: '',
                    ricevuta: '',
                    risposte: this.list_risposte,
                    tokenPrenotazione: this.ipResponse.tokenPrenotazione,
                };
                this.cptRequest = {
                    confermaPrenotazione: this.cpRequest,
                    dataInizioPrenotazione: this.addTimeZoneOffset(this.eventoSelezionato.startTime),
                    idUfficio: this.idUfficio,
                    nomeUfficio: this.nomeUfficio,
                    idRisorsa: this.uffOrRis.id,
                    nomeRisorsa: this.uffOrRis.nome,
                    spidCode: userInfo.identifier,
                    prenotazionePersonale: this.anagraficaFruitore.cf === this.anagraficaPresentante.cf ? 'Y' : 'N',
                };
                this.prenotazioneService.confermaPrenotazione(this.cptRequest).subscribe(
                    (result) => {
                        if (result.success) {
                            this.cpResponse = result.data.content;
                            if (this.cpResponse) {
                                this.audit(this.cpResponse.idPrenotazione);
                                this.showMessagePC();
                            }
                        }
                    },
                    (error) => {
                        this.globalService.presentToast('La prenotazione non è stata confermata', DURATIONTOAST.default);
                        //this.annullaPrenotazione(this.eventoSelezionato.uuid);
                    }
                );
            } else {
                this.globalService.presentToast('Impossibile completare la prenotazione', DURATIONTOAST.default);
            }
        });
    }

    annullaPrenotazione(uuid: string) {
        this.apRequest = {
            cfUser: this.coreRequest.cfUser,
            cfFruitore: this.coreRequest.cfFruitore,
            tokenPrenotazione: this.ipResponse.tokenPrenotazione,
        };
        this.prenotazioneService.annullaPrenotazione(this.apRequest).subscribe(
            (result) => {
                if (result.success) {
                    let message = result.data.content['message'];
                    this.setEventoEmpty();
                    this.setFlagSelezionato(uuid, false);
                }
            },
            (error) => {
                this.globalService.presentToast('La prenotazione non è stata annullata', DURATIONTOAST.default);
            }
        );
    }

    onTimeSelected(event) {
        if (this.calendar.mode === 'month' && !this.loadPeriodiDisponibili) {
            const isSelectableDate = moment(event.selectedTime).startOf('days').isSameOrAfter(new Date(), 'days');
            const isSameWeek = moment(event.selectedTime).isSame(new Date(), 'week');
            if (isSelectableDate || isSameWeek) {
                this.calendar.mode = 'week';
                this.showLegenda = true;
            }
            //TODO implementare il controllo la data selezionabile deve essere maggiore o uguale di oggi o nella settimana (verificare)
        }
    }

    controllaTitolo(titolo: string): boolean {
        let size = titolo.length;
        for (let i = 0; i < size; i++) {
            if (titolo[i] === ',') {
                return true;
            }
        }
        return false;
    }

    cambiaTitolo(titolo: string) {
        let mese_anno = titolo.split(',')[0];
        let numeroSettimana = titolo.split(',')[1].split('Week')[1];
        return mese_anno + ', settimana' + numeroSettimana;
    }

    setMinMaxPrenotabili() {
        if (this.infoRisorsa.giorniMaxPrenot && this.infoRisorsa.giorniMinPrenot) {
            let dayCurrent;

            this.dataMinPrenot = new Date();
            this.dataMinPrenot.setHours(0, 0, 0, 0);
            dayCurrent = this.dataMinPrenot.getDate();
            this.dataMinPrenot.setDate(this.infoRisorsa.giorniMinPrenot + dayCurrent);

            this.dataMaxPrenot = new Date();
            this.dataMaxPrenot.setHours(0, 0, 0, 0);
            dayCurrent = this.dataMinPrenot.getDate();
            this.dataMaxPrenot.setDate(this.infoRisorsa.giorniMaxPrenot + dayCurrent);
        } else {
            return;
        }
    }

    controllaMinMaxGiorniPrenotabili(evento: Evento): boolean {
        if (this.infoRisorsa.giorniMinPrenot && this.infoRisorsa.giorniMaxPrenot) {
            let dataMin = this.setMinPrenotabile(this.infoRisorsa.giorniMinPrenot);
            let dataMax = this.setMaxprenotabile(this.infoRisorsa.giorniMaxPrenot, dataMin.getDate());
            if (evento.startTime > dataMin && evento.startTime < dataMax) {
                return true;
            } else {
                return false;
            }
        } else if (this.infoRisorsa.giorniMinPrenot && !this.infoRisorsa.giorniMaxPrenot) {
            let dataMin = this.setMinPrenotabile(this.infoRisorsa.giorniMinPrenot);
            if (evento.startTime > dataMin) {
                return true;
            } else {
                return false;
            }
        } else if (!this.infoRisorsa.giorniMinPrenot && this.infoRisorsa.giorniMaxPrenot) {
            let dataMax = this.setMaxprenotabile(this.infoRisorsa.giorniMaxPrenot);
            if (evento.startTime < dataMax) {
                return true;
            } else {
                return false;
            }
        } else {
            return true;
        }
    }

    setMinPrenotabile(minGiorni: number): Date {
        let dayCurrent;
        this.dataMinPrenot = new Date();
        this.dataMinPrenot.setHours(0, 0, 0, 0);
        dayCurrent = this.dataMinPrenot.getDate();
        this.dataMinPrenot.setDate(minGiorni + dayCurrent);
        return this.dataMinPrenot;
    }

    setMaxprenotabile(maxGiorni: number, minGiorniAgg?: number): Date {
        if (maxGiorni && minGiorniAgg) {
            this.dataMaxPrenot = new Date();
            this.dataMaxPrenot.setHours(0, 0, 0, 0);
            this.dataMaxPrenot.setDate(maxGiorni + minGiorniAgg);
        } else {
            let dayCurrent;
            this.dataMaxPrenot = new Date();
            this.dataMaxPrenot.setHours(0, 0, 0, 0);
            dayCurrent = this.dataMaxPrenot.getDate();
            this.dataMaxPrenot.setDate(maxGiorni + dayCurrent);
        }
        return this.dataMaxPrenot;
    }

    async confermaOrAnnulla() {
        const alert = await this.alertController.create({
            cssClass: 'my-custom-class',
            header: '',
            message: 'Vuoi confermare la prenotazione per il ' + this.dataPrenotazione + ' ?',
            buttons: [
                {
                    text: 'Annulla',
                    handler: () => {
                        this.annullaPrenotazione(this.eventoSelezionato.uuid);
                    },
                },
                {
                    text: 'Conferma',
                    handler: () => {
                        this.confermaPrenotazione();
                    },
                },
            ],
        });
        await alert.present();
    }

    async showMessagePC() {
        const alert = await this.alertController.create({
            cssClass: 'my-custom-class',
            header: '',
            message: 'Prenotazione confermata',
        });
        await alert.present();
        setTimeout(() => {
            alert.dismiss();
            let pStep3Step4: PStep3Step4 = {
                coreRequest: this.coreRequest,
                nomeUfficio: this.nomeUfficio,
                uffOrRis: this.uffOrRis,
                nomeFruitore: this.anagraficaFruitore.nome,
                cognomeFruitore: this.anagraficaFruitore.cognome,
                dataPrenotazione: this.dataPrenotazione,
                b64PdfPrenotazione: this.cpResponse.b64PdfPrenotazione,
            };
            let params = this.globalService.encodeParams(pStep3Step4);
            this.router.navigate([PAGES.prenotazioneStep4.link, params], {
                relativeTo: this.activatedRoute,
            });
        }, 2000);
    }

    transformDateCompletToString(date: Date): string {
        if (date) {
            let day = date.getDate();
            let month = this.list_mesi[date.getMonth()];
            let year = date.getFullYear();
            let hours = date.getHours();
            let minutes;
            if (date.getMinutes() >= 0 && date.getMinutes() <= 9) {
                minutes = '0' + date.getMinutes();
            } else {
                minutes = date.getMinutes();
            }
            return `${day} ${month} ${year} alle ${hours}:${minutes}`;
        } else {
            return '';
        }
    }

    getValoriRisposta() {
        if (this.valoriRisposta) {
            for (let valoreRisposta of this.valoriRisposta) {
                this.rispostaTemp = {
                    codice: valoreRisposta.codice + '',
                    valore: valoreRisposta.valore,
                };
                this.list_risposte.push(this.rispostaTemp);
            }
        } else {
            return (this.list_risposte = []);
        }
    }

    transformDateToString(date: Date): string {
        if (date) {
            let hours;
            if (date.getHours() >= 0 && date.getHours() <= 9) {
                hours = '0' + date.getHours();
            } else {
                hours = date.getHours();
            }
            let minutes;
            if (date.getMinutes() >= 0 && date.getMinutes() <= 9) {
                minutes = '0' + date.getMinutes();
            } else {
                minutes = date.getMinutes();
            }
            return this.datePipe.transform(date, 'dd-MM-yyyy') + ', ' + hours + ':' + minutes;
        } else {
            return '';
        }
    }

    addSlotOccuppato(eventoOccuputo: Evento) {
        this.slotOccupatoTemp = {
            inizio: null,
            fine: null,
        };
        if (this.lista_slot_occupati.length === 0) {
            this.slotOccupatoTemp = {
                inizio: eventoOccuputo.startTime.toLocaleString(),
                fine: eventoOccuputo.endTime.toLocaleString(),
            };
            this.lista_slot_occupati.push(this.slotOccupatoTemp);
        } else {
            let trovato: boolean = false;
            for (let i = 0; i < this.lista_slot_occupati.length; i++) {
                if (this.lista_slot_occupati[i].inizio === eventoOccuputo.startTime.toLocaleString()) {
                    trovato = true;
                }
            }
            if (!trovato) {
                this.slotOccupatoTemp = {
                    inizio: eventoOccuputo.startTime.toLocaleString(),
                    fine: eventoOccuputo.endTime.toLocaleString(),
                };
                this.lista_slot_occupati.push(this.slotOccupatoTemp);
            }
        }
    }

    audit(id_prenotazione: number) {
        this.auditRequest = {
            t_nome_ufficio: this.nomeUfficio,
            n_id_ufficio: this.idUfficio,
            t_nome_risorsa: this.uffOrRis.nome,
            n_id_risorsa: this.uffOrRis.id,
            n_id_prenotazione: id_prenotazione,
            d_data_inizio_appuntamento: this.addTimeZoneOffset(this.eventoSelezionato.startTime),
            d_data_fine_appuntamento: this.addTimeZoneOffset(this.eventoSelezionato.endTime),
            d_data_prenotazione: this.addTimeZoneOffset(new Date()),
            t_questionario_compilato: this.questionarioNotObbligato,
            t_tipologia_prenotazione: this.anagraficaFruitore.cf === this.anagraficaPresentante.cf ? '0' : '1',
            t_slot_occupati: this.lista_slot_occupati,
        };
        this.prenotazioneService.audit(this.auditRequest).subscribe(
            (result) => { },
            (error) => {
                console.log('audit-erro', error);
            }
        );
    }

    addTimeZoneOffset(data: Date) {
        if (data.getTimezoneOffset() === -60) {
            let hours = data.getHours();
            data.setHours(hours + 1);
        }
        if (data.getTimezoneOffset() === -120) {
            let hours = data.getHours();
            data.setHours(hours + 2);
        }
        return data;
    }
}
