import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { ModalController, Platform } from '@ionic/angular';
import { Subscription } from 'rxjs';
import { pluck } from 'rxjs/operators';
import { CoreRequest, PStep3Step4, Risorsa } from 'src/app/models';
import { IMAGE, OPEN_WITH_EXTERNAL_BROWSER, PAGES } from 'src/app/utility';

import { ModalPdfViewerComponent } from '../../../components';
import { GlobalService } from '../../../services';

@Component({
    selector: 'app-prenotazione-step4',
    templateUrl: './prenotazione-step4.page.html',
    styleUrls: ['./prenotazione-step4.page.scss'],
})
export class PrenotazioneStep4Page implements OnInit {
    titleHeader: string = 'Prenotazione Step 4';
    logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
    coreRequest: CoreRequest;
    uffOrRis: Risorsa;
    nomeFruitore: string;
    cognomeFruitore: string;
    nomeUfficio: string;
    dataPrenotazione: string;
    b64PdfPrenotazione: string;
    routeSub$: Subscription;
    constructor(
        private globalService: GlobalService,
        private router: Router,
        private activatedRoute: ActivatedRoute,
        private modalController: ModalController,
        private platform: Platform,
        private inapp: InAppBrowser
    ) {
        this.getParams();
        this.routeSub$ = this.router.events.subscribe((res) => {
            if ((res as any)?.url?.includes('p-step3')) {
                this.routeSub$?.unsubscribe();
                this.router.navigateByUrl(PAGES.home_page);
            }
        });
    }

    ngOnInit() {}

    getParams() {
        this.activatedRoute.params.pipe(pluck('params')).subscribe((result) => {
            let pStep3Step4: PStep3Step4;
            pStep3Step4 = this.globalService.decodeParams(result);
            this.coreRequest = pStep3Step4.coreRequest;
            this.nomeUfficio = pStep3Step4.nomeUfficio;
            this.uffOrRis = pStep3Step4.uffOrRis;
            this.nomeFruitore = pStep3Step4.nomeFruitore;
            this.cognomeFruitore = pStep3Step4.cognomeFruitore;
            this.dataPrenotazione = pStep3Step4.dataPrenotazione;
            this.b64PdfPrenotazione = pStep3Step4.b64PdfPrenotazione;
        });
    }

    scaricaRicevuta() {
        let fileURL = URL.createObjectURL(this.globalService.b64toBlob(this.b64PdfPrenotazione, 'application/pdf'));

        if (this.platform.is('ios')) {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        } else {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        }
    }

    goToMenu() {
        this.router.navigateByUrl(PAGES.home_page);
        setTimeout(() => {
            this.router.navigateByUrl(PAGES.menu);
        }, 100);
    }

    async openPdfViewrModal() {
        const blob = this.globalService.b64toBlob(this.b64PdfPrenotazione, 'application/pdf');
        let fileURL = URL.createObjectURL(blob);
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: fileURL,
                base64File: blob,
                fileName: `Prenotazione_${new Date().getTime()}.pdf`,
            },
        });
        return await modal.present();
    }
}
