import { DatePipe, Location } from '@angular/common';
import { Component, OnInit } from '@angular/core';
import { FormControl } from '@angular/forms';
import { InAppBrowser } from '@ionic-native/in-app-browser/ngx';
import { AlertController, ModalController, Platform } from '@ionic/angular';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';
import { GlobalService, PrenotazioneService, StorageService } from 'src/app/services';
import { OPEN_WITH_EXTERNAL_BROWSER } from 'src/app/utility';
import { DURATIONTOAST, MESSAGES } from 'src/app/utility/global-constant';

import { ModalPdfViewerComponent, PsDateRangerComponent } from '../../../components';
import { CoreRequest, DateRange, DettaglioCardPrenotazione, DisdiciPrenotazioneRequest, Prenotazione, PrenotazioneRequest } from '../../../models';
import { IMAGE, RESERVATION } from '../../../utility/global-constant';

@Component({
    selector: 'app-storico-prenotazione',
    templateUrl: './storico-prenotazione.page.html',
    styleUrls: ['./storico-prenotazione.page.scss'],
})
export class StoricoPrenotazionePage implements OnInit {
    titleHeader: string = 'Elenco prenotazioni';
    showSearchBar: boolean;
    select: string;
    arrow_down: string = 'assets/icon/icona_sotto.svg';
    arrow_right: string = 'assets/icon/icona_avanti.svg';
    list_dettaglio_card_prenotazione: Array<DettaglioCardPrenotazione>;
    list_prenotazioni: Array<Prenotazione>;
    dcPrenotazione: DettaglioCardPrenotazione;
    coreRequest: CoreRequest;
    dpRequest: DisdiciPrenotazioneRequest;
    public searchTerm: string = '';
    filterFeeds: Prenotazione[];
    today: number = Date.now();
    viewTitle;
    pipe = new DatePipe('it-IT');
    initialDate: any;
    finalDate: any;
    change: string;
    change2: string;
    alertButton: boolean;
    list_filterData: any = [];
    saveData: any;
    zeroPrenotazioni: boolean;
    filtroScelto: string;
    showIFDate: boolean;
    showSelectCF: boolean;
    showInputForOthers: boolean;
    showButtonSearchForYouOrOthers: boolean;
    showCfUpdate: boolean;
    radioScelto: string;
    isChoosed: boolean;
    inputCF: string;
    maxYear: number;
    isRefreshing: boolean;
    forMeValue: string = RESERVATION.forMeValue;
    forMeLabel: string = RESERVATION.forMeLabel;
    forOthersValue: string = RESERVATION.forOthersValue;
    forOthersLabel: string = RESERVATION.forOthersLabel;
    checkChoose: string = RESERVATION.forMeValue;
    searchIcon: string = 'assets/icon/icona_storico_prenotazione.svg';
    reloadIcon: string = 'assets/icon/icona_ricarica.svg';
    ios: boolean;
    android: boolean;
    logoHeader: string = IMAGE.logoHeader;
    widhtLogo: number = IMAGE.widthLogo;
    heightLogo: number = IMAGE.heightLogo;
    otherCf: FormControl;
    dateRange: DateRange;
    showBackdrop: boolean = false;
    refresherRef: CustomEvent;

    constructor(
        public location: Location,
        private prenotazioneService: PrenotazioneService,
        private globalService: GlobalService,
        public alertController: AlertController,
        private platform: Platform,
        private inapp: InAppBrowser,
        private storageService: StorageService,
        private modalController: ModalController) {
        this.otherCf = new FormControl('');
        this.otherCf.valueChanges.pipe(debounceTime(500), distinctUntilChanged()).subscribe((res) => {
            if (this.globalService.checkCF(res)) {
                this.ottieniPrenotazioni();
            } else {
                this.list_prenotazioni = null;
                this.filterFeeds = null;
            }
        });
        this.setUtility();
        this.checkPlatform();
        this.list_prenotazioni = [];
    }

    ngOnInit() { }

    setVariabili() {
        this.isChoosed = false;
        this.showIFDate = false;
        this.showSearchBar = false;
        this.showSelectCF = true;
        this.showInputForOthers = false;
        this.showButtonSearchForYouOrOthers = true;
        this.showCfUpdate = false;
        this.checkChoose = this.forMeValue;
        this.selezionaPC(null, true);
    }

    ionViewWillEnter() {
        this.list_prenotazioni = [];
        this.isRefreshing = false;
        this.setVariabili();
    }

    setUtility() {
        this.maxYear = this.getYearUpdate();
        this.coreRequest = {
            cfFruitore: '',
            cfUser: '',
        };
    }

    setListe() {
        this.list_dettaglio_card_prenotazione = [];
        for (let i = 0; i < this.list_prenotazioni.length; i++) {
            this.dcPrenotazione = {
                arrow: this.arrow_right,
                isOpen: false,
            };
            this.list_dettaglio_card_prenotazione.push(this.dcPrenotazione);
        }
    }

    openOrCloseCard(i: number) {
        if (this.list_dettaglio_card_prenotazione[i].isOpen) {
            this.list_dettaglio_card_prenotazione[i].isOpen = false;
            this.list_dettaglio_card_prenotazione[i].arrow = this.arrow_right;
        } else {
            this.list_dettaglio_card_prenotazione[i].isOpen = true;
            this.list_dettaglio_card_prenotazione[i].arrow = this.arrow_down;
        }
    }

    getPrenotazioni() {
        this.prenotazioneService.getPrenotazioni(this.coreRequest).subscribe(
            (result) => {
                if (result.success) {
                    this.list_prenotazioni = result.data.content.prenotazioni;
                    if (this.list_prenotazioni && this.list_prenotazioni.length > 0) {
                        this.filterPrenotazioniByCf();
                        this.orderByDate();

                        this.filterFeeds = JSON.parse(JSON.stringify(this.list_prenotazioni));
                        this.setListe();
                        this.setPrenotationFromCurrentToFuture();

                        // TODO this.setPrenotationFromCurrentToFuture();
                        this.zeroPrenotazioni = false;
                    } else {
                        this.zeroPrenotazioni = true;
                    }
                }
            },
            (error) => {
                this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
                this.zeroPrenotazioni = true;
            },
            () => {
                if (this.refresherRef) {
                    this.refresherRef.detail.complete();
                }
            }
        );
    }

    onViewTitleChanged(title) {
        this.viewTitle = title;
    }

    disdiciPrenotazione(prenotazione: Prenotazione) {
        this.dpRequest = {
            cfFruitore: prenotazione.anagraficaFruitore.cf,
            cfUser: prenotazione.anagraficaPresentante.cf,
            idPrenotazione: prenotazione.idPrenotazione,
        };
        /*  this.loader.presentLoader(); */
        this.prenotazioneService.disdiciPrenotazione(this.dpRequest).subscribe(
            (result) => {
                /*    this.loader.dismissLoader(); */
                if (result.success) {
                    this.globalService.presentToast('La prenotazione è stata disdetta', DURATIONTOAST.default);
                    this.getPrenotazioni();
                } else {
                    this.globalService.presentToast('La prenotazione non è stata disdetta', DURATIONTOAST.default);
                }
            },
            (error) => {
                /*   this.loader.dismissLoader(); */
                this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
            }
        );
    }

    async confermaOrAnnullaDisdetta(prenotazione: Prenotazione) {
        const alert = await this.alertController.create({
            cssClass: 'my-custom-class',
            header: '',
            message: 'Sei sicuro di voler disdire ?',
            buttons: [
                {
                    text: 'No',
                    role: 'cancel',
                },
                {
                    text: 'Si',
                    role: 'ok',
                    handler: () => {
                        this.disdiciPrenotazione(prenotazione);
                    },
                },
            ],
        });
        await alert.present();
    }

    showCalendarInitial(dateInitial) {
        const day = dateInitial.day;
        const month = dateInitial.month;
        const year = dateInitial.year;
        let from = new Date(year, month - 1, day);
        const myFormattedDate1 = this.pipe.transform(from, 'yyyy-MM-dd');
        this.initialDate = myFormattedDate1;
        return this.initialDate;
    }

    showCalendarFinal(dateFinal) {
        const day: number = dateFinal.day;
        const month: number = dateFinal.month;
        const year: number = dateFinal.year;
        let to = new Date(year, month - 1, day);
        const myFormattedDate2 = this.pipe.transform(to, 'yyyy-MM-dd');
        this.finalDate = myFormattedDate2;
        return this.finalDate;
    }

    scaricaRicevuta(pdfBase64: string) {
        let blob = this.globalService.b64toBlob(pdfBase64, 'application/pdf');
        let fileURL = URL.createObjectURL(blob);
        if (this.platform.is('ios')) {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        } else {
            this.inapp.create(fileURL, OPEN_WITH_EXTERNAL_BROWSER ? '_system' : '_blank', 'hideurlbar=yes,location=no,toolbar=no,enableViewportScale=yes');
        }
    }

    getYearUpdate(): number {
        let dateNow: Date = new Date();
        return dateNow.getFullYear() + 1;
    }

    closeAllCards() {
        if (this.list_dettaglio_card_prenotazione) {
            for (let dc of this.list_dettaglio_card_prenotazione) {
                dc.isOpen = false;
            }
        }
    }

    selezionaPC(event: any, forced?: boolean) {
        let choose: string = forced ? this.forMeValue : event?.detail?.value;
        this.checkChoose = choose;
        this.list_prenotazioni = [];
        this.filterFeeds = [];
        this.dateRange = null;
        if (choose === this.forMeValue) {
            this.ottieniPrenotazioni();
        }
    }

    ottieniPrenotazioni() {
        if (this.checkChoose) {
            if (this.checkChoose === this.forOthersValue && !this.globalService.checkCF(this.otherCf.value)) {
                this.globalService.presentToast(MESSAGES.checkFieled, DURATIONTOAST.default);
                return;
            }
            this.storageService.getUserInfo().then((ui) => {
                if (ui) {
                    this.showButtonSearchForYouOrOthers = true;
                    this.coreRequest = {
                        cfFruitore:
                            this.checkChoose === this.forOthersValue ? (this.otherCf.value as string).toLocaleUpperCase() : ui.identifier.toLocaleUpperCase(),
                        cfUser: ui.identifier.toLocaleUpperCase(),
                    };
                    this.isChoosed = true;
                    this.showCfUpdate = true;
                    this.getPrenotazioni();
                } else {
                    this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
                }
            });
        }
    }

    changeCF() {
        this.radioScelto = this.forMeValue;
        this.inputCF = '';
        this.isChoosed = false;
        this.showInputForOthers = false;
        this.list_prenotazioni = [];
        this.filterFeeds = [];
        this.searchTerm = '';
        this.filtroScelto = '';
        this.showSearchBar = false;
        //this.showCheckBox = false;
        this.showIFDate = false;
        this.showCfUpdate = false;
        this.zeroPrenotazioni = false;
    }

    doRefresh(refresher) {
        this.refresherRef = refresher;
        this.list_prenotazioni = this.filterFeeds = [];
        this.dateRange = null;
        this.getPrenotazioni();
    }

    checkPlatform() {
        this.ios = this.platform.is('ios');
        this.android = this.platform.is('android');
    }

    async rangeDate() {
        this.showBackdrop = true;
        const modal = await this.modalController.create({
            component: PsDateRangerComponent,
            showBackdrop: true,
            keyboardClose: false,
            cssClass: 'modalRangeDate',
        });
        modal.onWillDismiss().then((response) => {
            this.showBackdrop = false;
            if (response.data) {
                this.dateRange = response.data;
                this.list_prenotazioni = this.filterFeeds.filter((lp) => {
                    const slotStart = this.getDateWithoutHours(new Date(Number(lp.slot[0].inizio))).getTime();
                    const filterStart = this.getDateWithoutHours(this.dateRange.from.toDate()).getTime();
                    const slotEnd = this.getDateWithoutHours(new Date(Number(lp.slot[0].fine))).getTime();
                    const filterEnd = this.getDateWithoutHours(this.dateRange.to.toDate()).getTime();
                    return slotStart >= filterStart && slotEnd <= filterEnd;
                });
            } else {
                console.log('NESSUN RANGE');
            }
        });
        return await modal.present();
    }

    removeRangeDate() {
        this.dateRange = null;
        this.resetFilter();
    }

    resetFilter() {
        this.list_prenotazioni = this.filterFeeds;
        this.setPrenotationFromCurrentToFuture();
        // this.filterFeeds = this.list_prenotazioni;
    }

    getDateWithoutHours(date: Date) {
        date.setHours(0);
        date.setMinutes(0);
        date.setSeconds(0);
        date.setMilliseconds(0);
        return date;
    }

    setPrenotationFromCurrentToFuture() {
        this.list_prenotazioni = this.filterFeeds.filter((lp) => {
            const slotStart = this.getDateWithoutHours(new Date(Number(lp.slot[0].inizio))).getTime();
            const filterStart = this.getDateWithoutHours(new Date()).getTime();
            return slotStart >= filterStart;
        });
    }

    orderByDate() {
        if (this.list_prenotazioni) {
            this.list_prenotazioni = this.list_prenotazioni.sort((ffa, ffb) => Number(ffa.slot[0].inizio) - Number(ffb.slot[0].inizio));
        }
    }

    showReceived(prenotazione: Prenotazione) {
        this.getDettaglioPrenotazione(prenotazione);
    }

    getDettaglioPrenotazione(prenotazione: Prenotazione) {
        this.storageService.getUserInfo().then((userInfo) => {
            if (userInfo) {
                let prenotazioneRequest: PrenotazioneRequest = {
                    cfFruitore:
                        this.checkChoose === this.forOthersValue ? (this.otherCf.value as string).toLocaleUpperCase() : userInfo.identifier.toLocaleUpperCase(),
                    cfUser: userInfo.identifier.toLocaleUpperCase(),
                    idPrenotazione: prenotazione.idPrenotazione,
                };
                this.prenotazioneService.getPrenotazioneByID(prenotazioneRequest).subscribe(
                    (result) => {
                        if (result.success) {
                            if (result.data.content && result.data.content.prenotazioni && result.data.content.prenotazioni.length > 0) {
                                let pdfBase64: string = result.data.content.prenotazioni[0].b64PdfPrenotazione;
                                pdfBase64
                                    ? this.openPdfViewrModal(pdfBase64, prenotazione.idPrenotazione)
                                    : this.globalService.presentToast(MESSAGES.notAvaiblePDF, DURATIONTOAST.default);
                            }
                        } else {
                            this.globalService.presentToast(MESSAGES.notAvaiblePDF, DURATIONTOAST.default);
                        }
                    },
                    () => {
                        this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
                        this.zeroPrenotazioni = true;
                    }
                );
            } else {
                this.globalService.presentToast(MESSAGES.notAvaiblePDF, DURATIONTOAST.default);
            }
        });
    }

    async openPdfViewrModal(pdfBase64: string, idPrenotazione: number) {
        let blob = this.globalService.b64toBlob(pdfBase64, 'application/pdf');
        let fileURL = URL.createObjectURL(blob);
        const modal = await this.modalController.create({
            component: ModalPdfViewerComponent,
            showBackdrop: true,
            componentProps: {
                pdfSrc: fileURL,
                base64File: blob,
                fileName: `Prenotazione n°${idPrenotazione}.pdf`,
            },
        });
        return await modal.present();
    }

    filterPrenotazioniByCf() {
        if (this.list_prenotazioni) {
            this.list_prenotazioni = this.list_prenotazioni.filter(
                (lp) => lp.anagraficaFruitore.cf === this.coreRequest.cfFruitore && lp.anagraficaPresentante.cf === this.coreRequest.cfUser
            );
        }
    }

    isDisdicibileFromTime(prenotazione: Prenotazione): boolean {
        let result = false;
        try {
            result = Number(prenotazione?.slot[0]?.inizio) >= new Date().getTime() ?  true: false;
        } catch(e) {
            result = false;
        }
        return result;
    }
}

//CPRNDR80A01A783N
