import { Component, OnDestroy, OnInit } from '@angular/core';
import { NavController } from '@ionic/angular';
import { AuthActions, AuthService, IAuthAction } from 'ionic-appauth';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { Subscription } from 'rxjs';
import { IdpInformationService } from 'src/app/services';
import { AuthenticationService } from 'src/app/services/authentication/authentication.service';
import { CONSOLE, PAGES, TEXT } from 'src/app/utility';

import { IMAGE } from '../../../utility/global-constant';

@Component({
	selector: 'app-landing',
	templateUrl: './landing.page.html',
	styleUrls: ['./landing.page.scss'],
})
export class LandingPage implements OnInit, OnDestroy {
	events$ = this.auth.events$;
	sub: Subscription;
	titleHeader: string = 'Home';
	titleText: string = TEXT.home.title;
	buttonText: string = TEXT.home.buttonEntra;
	countCheckLogout: number;
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;

	constructor(
		private auth: AuthService,
		private authenticationService: AuthenticationService,
		private navCtrl: NavController,
		private ngxService: NgxUiLoaderService,
		private idpInformationService: IdpInformationService
	) {
		this.setUtility();
	}

	ngOnInit() {
		this.auth.loadTokenFromStorage();
		this.sub = this.auth.events$.subscribe((action) => this.onSignInSuccess(action));
	}

	ngOnDestroy() {
		this.sub.unsubscribe();
	}

	setUtility() {
		this.countCheckLogout = 0;
	}

	private onSignInSuccess(action: IAuthAction) {
		if (action.action === AuthActions.SignInSuccess) {
			this.authenticationService.login();
			setTimeout(() => {
				this.idpInformationService.saveUserInformation();
			});
			this.navCtrl.navigateRoot(PAGES.home_page);
		} else if (
			//  action.action === AuthActions.LoadTokenFromStorageFailed ||
			action.action === AuthActions.LoadUserInfoFailed ||
			action.action === AuthActions.SignInFailed ||
			action.action === AuthActions.RefreshFailed
		) {
			this.checkIsLoggedAndLogout();
		}
	}

	public login() {
		this.startAndStopLoader(true);
		this.auth.signIn().then(() =>
			setTimeout(() => {
				this.startAndStopLoader();
			}, 1000)
		);
	}

	startAndStopLoader(start?: boolean) {
		start ? CONSOLE.log && console.log('[LandingPage - startAndStopLoader - start]') : CONSOLE.log && console.log('[LandingPage - startAndStopLoader - stop]');
		start ? this.ngxService.start() : this.ngxService.stop();
	}

	checkIsLoggedAndLogout() {
		this.authenticationService.isLoggedIn().then((autenticato) => {
			if (autenticato && this.countCheckLogout === 0) {
				this.countCheckLogout++;
				this.authenticationService.logout();
			}
		});
	}
}
