import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Platform } from '@ionic/angular';
import { AuthService } from 'ionic-appauth';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { GlobalService, StorageService, UtilityService } from 'src/app/services';
import { AuthenticationService } from 'src/app/services/authentication/authentication.service';
import { AUTH_LOGOUT } from 'src/app/utility';
import { DURATIONTOAST, IMAGE, MESSAGES, PAGES, PATHWP } from 'src/app/utility/global-constant';

import { DettaglioUtente, Funzionalita, Funzione } from '../../../models';

@Component({
	selector: 'app-menu',
	templateUrl: './menu.page.html',
	styleUrls: ['./menu.page.scss'],
})
export class MenuPage implements OnInit {
	titleHeader: string = 'Menu';
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	dettaglioUtente: DettaglioUtente;
	listaFunzioniDefault: Array<Funzione>;
	funzioni_utenza: Array<Funzione>;
	id_user: string;
	isAutenticato: boolean;
	size_array_pipe: number = 3;
	icon_default: string;
	listOpenSottoMenu: Array<boolean>;
	listOpenSottoInfoMenu: Array<boolean>;
	notShowAcccount: boolean;
	notShowInfo: boolean;
	ios: boolean;
	android: boolean;

	constructor(
		private platform: Platform,
		private storageService: StorageService,
		private globalService: GlobalService,
		private authService: AuthenticationService,
		private router: Router,
		private activatedRoute: ActivatedRoute,
		private cdr: ChangeDetectorRef,
		private auth: AuthService,
		private ngxService: NgxUiLoaderService,
		private utilityService: UtilityService
	) {
		this.checkPlatform();
		this.setUtility();
		this.subscriptionLoggedIN();
		this.subscribeUpdateProfilazione();
	}

	ngOnInit() {}

	ionViewWillEnter() {
		if (this.isAutenticato) {
			this.funzioni_utenza = null;
			setTimeout(() => {
				this.getDettaglioUtenza();
			}, 500);
		} else {
			this.funzioni_utenza = [];
		}
	}

	ngAfterViewInit() {
		this.cdr.detectChanges();
	}

	setUtility() {
		this.isAutenticato = false;
		this.icon_default = 'macs-icon-temporanea'; //TODO
		this.notShowAcccount = false;
		this.setMockDeafult();
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	setMockDeafult() {
		this.listaFunzioniDefault = [
			{
				n_id_funzione: 1,
				t_nome: 'Help',
				t_descrizione: '',
				d_data_ins: null,
				funzionalita: [
					{
						n_id_funzionalita: 1,
						t_nome: 'Tutorial',
						t_descrione: '',
						d_data: null,
						t_livello_sicurezza: '0',
						t_info_aggiuntive: '/tabs/menu/tutorial|icona_tutorial.svg|menu',
					},
				],
			},
		];
		this.setListaOpenSottoInfoMenu();
	}

	goToPage(path: string) {
		if (path && path !== '') {
			if (PATHWP.tutorial === path) {
				let params = this.globalService.encodeParams({ t: 't' });
				this.router.navigate([path, params], { relativeTo: this.activatedRoute });
			} else {
				this.router.navigate([path], { relativeTo: this.activatedRoute });
			}
		} else {
			this.globalService.presentToast(MESSAGES.notAuthorized, DURATIONTOAST.default);
		}
	}

	login() {
		this.router.navigate([PAGES.landing_page], { relativeTo: this.activatedRoute });
	}

	logout() {
		this.startAndStopLoader(true);
		this.auth.signOut('', AUTH_LOGOUT.revokeToken).then(() => {
			setTimeout(() => {
				this.startAndStopLoader();
				this.dettaglioUtente = null;
				this.isAutenticato = false;
				this.authService.logout();
			}, 1200);
		});
	}

	getPage(info: string): string {
		let array_pipe = info.split('|');
		if (array_pipe.length === this.size_array_pipe) {
			return array_pipe[0];
		} else {
			return null;
		}
	}

	getIcon(info: string): string {
		let array_pipe = info.split('|');
		if (array_pipe.length === this.size_array_pipe) {
			if (array_pipe[1] !== '') {
				return 'assets/icon/' + array_pipe[1];
			} else {
				return this.icon_default;
			}
		} else {
			return this.icon_default;
		}
	}

	checkIsForMenu(info: string): boolean {
		let array_pipe = info.split('|');
		if (array_pipe.length === this.size_array_pipe) {
			if (array_pipe[2] !== '' && array_pipe[2] === 'menu') {
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	setListaOpenSottoMenu() {
		if (this.funzioni_utenza.length > 0) {
			this.listOpenSottoMenu = [];
			for (let i = 0; i < this.funzioni_utenza.length; i++) {
				//TUTTO APERTO
				this.listOpenSottoMenu.push(true);
			}
		} else {
			return;
		}
	}

	setListaOpenSottoInfoMenu() {
		if (this.listaFunzioniDefault.length > 0) {
			this.listOpenSottoInfoMenu = [];
			for (let i = 0; i < this.listaFunzioniDefault.length; i++) {
				//TUTTO APERTO
				this.listOpenSottoInfoMenu.push(true);
			}
		} else {
			return;
		}
	}

	apriSottoMenu(index: number) {
		if (this.listOpenSottoMenu[index]) {
			this.listOpenSottoMenu[index] = false;
		} else {
			this.listOpenSottoMenu[index] = true;
		}
	}

	apriSottoMenuAccount() {
		if (this.notShowAcccount) {
			this.notShowAcccount = false;
		} else {
			this.notShowAcccount = true;
		}
	}

	apriSottoMenuInfo(index: number) {
		if (this.listOpenSottoInfoMenu[index]) {
			this.listOpenSottoInfoMenu[index] = false;
		} else {
			this.listOpenSottoInfoMenu[index] = true;
		}
	}

	closeAllSottoMenu() {
		for (let i = 0; i < this.listOpenSottoMenu.length; i++) {
			this.listOpenSottoMenu[i] = false;
		}
	}

	openAllSottoMenu() {
		for (let i = 0; i < this.listOpenSottoMenu.length; i++) {
			this.listOpenSottoMenu[i] = true;
		}
	}

	subscriptionLoggedIN() {
		this.authService.loggedIN$.subscribe({
			next: (value) => {
				this.isAutenticato = value;
			},
			error: () => {},
		});
	}

	getDettaglioUtenza() {
		this.storageService.getDettaglioUtenza().then((dettaglioUtenza) => {
			if (dettaglioUtenza) {
				this.funzioni_utenza = dettaglioUtenza.ruolo.funzioni;
				this.reOrderFunctionForMenu();
				this.setListaOpenSottoMenu();
				this.isAutenticato = true;
			} else {
				this.isAutenticato = false;
			}
		});
	}

	reOrderFunctionForMenu() {
		let funzioni: Funzione[] = [];
		let funzionilitaForMenu: Funzionalita[] = [];
		this.funzioni_utenza.forEach((funzione) => {
			funzione.funzionalita.forEach((funzionalita) => {
				if (this.checkIsForMenu(funzionalita?.t_info_aggiuntive)) funzionilitaForMenu.push(funzionalita);
			});
			if (funzionilitaForMenu.length > 0) {
				funzione.funzionalita = funzionilitaForMenu;
				funzioni.push(funzione);
			}
			funzionilitaForMenu = [];
		});

		this.funzioni_utenza = funzioni;
		//console.log('[MenuPage - reOrderFunctionForMenu - funzioni]: ', this.funzioni_utenza);
	}

	startAndStopLoader(start?: boolean) {
		start ? console.log('[LandingPage - startAndStopLoader - start]') : console.log('[LandingPage - startAndStopLoader - stop]');
		start ? this.ngxService.start() : this.ngxService.stop();
	}

	subscribeUpdateProfilazione() {
		this.utilityService.updateProfilazione$.subscribe((status) => {
			if (status) {
				setTimeout(() => {
					this.getDettaglioUtenza();
				}, 500);
			}
		});
	}
}
