import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Allegato, NewsEventiDettaglio, NewsEventiRequest, NewsEventiResponse } from 'src/app/models';
import { PORTALE_ISTITUZIONALE_API } from 'src/app/utility';
import { IMAGE } from 'src/app/utility/global-constant';

import { htmlCodeToChar } from '../../utility/character';
import { IMAGE_EXTENSION } from '../../utility/global-constant';

export const NEWS: string = '/json/mobile/news';
export const PP: string = '/json/mobile/primopiano';
export const COM: string = '/json/mobile/comunicati';
export const AVV: string = '/json/mobile/avvisi';
export const DNPCA: string = '/json/mobile/dettaglio/';
export const PPH: string = '/json/mobile/primopiano-home';

@Injectable({
	providedIn: 'root',
})
export class NewsEventiService {
	imageTemp: string = IMAGE.neLocandinaTemp;
	constructor(private httpClient: HttpClient) {}

	getNewsPaginated(neRequest: NewsEventiRequest): Observable<NewsEventiResponse> {
		let items_per_page = '?items_per_page=' + neRequest.items_per_page;
		let page = '&page=' + neRequest.page;
		let title = '&title=' + neRequest.title;
		let parametri = items_per_page + page + title;
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${NEWS}${parametri}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows = this.buildPathImage(response.rows);
				return response;
			})
		);
	}

	getPrimoPianoPaginated(neRequest: NewsEventiRequest): Observable<NewsEventiResponse> {
		let items_per_page = '?items_per_page=' + neRequest.items_per_page;
		let page = '&page=' + neRequest.page;
		let title = '&title=' + neRequest.title;
		let parametri = items_per_page + page + title;
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${PP}${parametri}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows = this.buildPathImage(response.rows);
				return response;
			})
		);
	}

	getComunicatiPaginated(neRequest: NewsEventiRequest): Observable<NewsEventiResponse> {
		let items_per_page = '?items_per_page=' + neRequest.items_per_page;
		let page = '&page=' + neRequest.page;
		let title = '&title=' + neRequest.title;
		let parametri = items_per_page + page + title;
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${COM}${parametri}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows = this.buildPathImage(response.rows);
				return response;
			})
		);
	}

	getAvvisiPaginated(neRequest: NewsEventiRequest): Observable<NewsEventiResponse> {
		let items_per_page = '?items_per_page=' + neRequest.items_per_page;
		let page = '&page=' + neRequest.page;
		let title = '&title=' + neRequest.title;
		let parametri = items_per_page + page + title;
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${AVV}${parametri}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows = this.buildPathImage(response.rows);
				return response;
			})
		);
	}

	//N news, P primopiano, C comunicati, A avvisi
	getDettaglioNPCA(id_npca: string): Observable<NewsEventiResponse> {
		let parametri = id_npca;
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${DNPCA}${parametri}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows[0].field_image = this.checkSetPathImage(response.rows[0]?.field_image, response?.rows[0]?.base_url);
				response.rows[0].listaImage = this.getListaImage(response?.rows[0]?.field_galleria_di_immagini + ',' + response?.rows[0]?.field_multimedia, response?.rows[0]?.base_url);
				response.rows[0].listaVideoLocali = this.getListaVideo(response?.rows[0]?.field_multimedia, true, response?.rows[0]?.base_url);
				response.rows[0].listaVideoEsterni = this.getListaVideo(response?.rows[0]?.field_multimedia, false, response?.rows[0]?.base_url);
				response.rows[0].listaAllegati = this.getListaAllegati(response?.rows[0]?.field_documenti_allegati, response?.rows[0]?.base_url);
				response.rows[0].relatedList = this.parseCorrelated(response?.rows[0]?.field_correlati_documenti, response?.rows[0]?.base_url);
				response.rows[0].parsedACareOf = this.parseCorrelated(response?.rows[0]?.a_cura_di, response?.rows[0]?.base_url);
				response.rows[0].parsedCorrelatedNews = this.parseCorrelated(response?.rows[0]?.correlati_novita, response?.rows[0]?.base_url);
				response.rows[0].parsedCorrelatedPeople = this.parseCorrelated(response?.rows[0]?.correlati_persone, response?.rows[0]?.base_url);
				response.rows[0].parsedCorrelatedPlaces = this.parseCorrelated(response?.rows[0]?.correlati_luoghi, response?.rows[0]?.base_url);
				return response;
			})
		);
	}

	getHeadersDrupal(): HttpHeaders {
		return new HttpHeaders({
			/*       'Authorization': 'Basic ' + btoa(USERNAME_DRUPAL + ':' + PWD_DRUPAL),
			'Content-Type': 'application/json',
			'Access-Control-Allow-Origin': 'GET, POST' */
		});
	}

	checkSetPathImage(linkImage: string, base_url: string): string {
		if (linkImage && linkImage !== '') {
			if (!linkImage.includes('http')) {
				if (base_url && base_url !== '') {
					linkImage = base_url.trim() + '/' + linkImage;
				} else {
					linkImage = IMAGE.neLocandinaTemp;
				}
			}
		} else {
			linkImage = IMAGE.neLocandinaTemp;
		}
		return linkImage;
	}

	getListaImage(listaImageS: string, base_url: string): Array<string> {
		let listaImage: Array<string> = [];
		if (listaImageS && listaImageS !== '') {
			listaImageS.split(',').forEach((relativePathImage) => {
				if (relativePathImage && relativePathImage !== '') {
					const isValidExt = IMAGE_EXTENSION.some((ie) => {
						const ext = relativePathImage.slice(relativePathImage.length - ie.length);
						return ext.includes(ie);
					});
					if (isValidExt) {
						if (!relativePathImage.includes('http')) {
							if (base_url && base_url !== '') {
								listaImage.push(base_url.trim() + '/' + relativePathImage.trim());
							}
						} else {
							listaImage.push(relativePathImage.trim());
						}
					}
				}
			});
		}
		return listaImage;
	}

	getListaVideo(listaVideoS: string, isLocal: boolean, base_url: string): Array<string> {
		console.log('base_url:', base_url);
		let listaVideo: Array<string> = [];
		if (listaVideoS && listaVideoS !== '') {
			listaVideoS.split(',').forEach((relativePathVideo) => {
				const isValidExt = IMAGE_EXTENSION.some((ie) => {
					const ext = relativePathVideo.slice(relativePathVideo.length - ie.length);
					return ext.includes(ie);
				});
				if (!isValidExt) {
					if (!isLocal && relativePathVideo.includes('http') && (relativePathVideo.includes('youtube') || relativePathVideo.includes('youtu.be'))) {
						let idYoutube;
						try {
							if (relativePathVideo.includes('youtu.be')) {
								idYoutube = relativePathVideo.split('/')[3];
								idYoutube && listaVideo.push(idYoutube.trim());
							} else {
								idYoutube = relativePathVideo.split('=')[1];
								idYoutube && listaVideo.push(idYoutube.trim());
							}
						} catch (e) {}
					} else if (isLocal && !relativePathVideo.includes('http')) {
						base_url && base_url !== '' && listaVideo.push(base_url.trim() + '/' + relativePathVideo.trim());
					}
				}
			});
		}
		return listaVideo;
	}

	getListaAllegati(listaAllegatiSTR: string, base_url: string): Array<Allegato> {
		let listaAllegati: Array<Allegato> = [];
		if (listaAllegatiSTR !== '') {
			let firstSplit: Array<string>;
			firstSplit = listaAllegatiSTR.split('|');
			firstSplit.forEach((strComp) => {
				if (base_url && base_url !== '') {
					listaAllegati.push({
						nome: this.getNomeAllegato(strComp),
						url: this.getUrlAllegatoOrImage(strComp, base_url),
					});
				}
			});
		}
		return listaAllegati;
	}

	getUrlAllegatoOrImage(allegatoSTR: string, base_url: string) {
		return base_url.trim() + '/' + allegatoSTR.split('[')[0].trim();
	}

	getNomeAllegato(allegatoSTR: string) {
		return allegatoSTR.split('[')[1].replace(']', '');
	}

	buildPathImage(listaNews: Array<NewsEventiDettaglio>): Array<NewsEventiDettaglio> {
		if (listaNews && listaNews.length > 0) {
			listaNews.forEach((newsEventi) => {
				if (newsEventi.field_image && newsEventi.field_image !== '') {
					if (!newsEventi.field_image.includes('http')) {
						if (newsEventi.base_url && newsEventi.base_url !== '') {
							newsEventi.field_image = newsEventi.base_url.trim() + '/' + newsEventi.field_image;
						} else {
							newsEventi.field_image = IMAGE.neLocandinaTemp;
						}
					}
				} else {
					newsEventi.field_image = IMAGE.neLocandinaTemp;
				}
			});
		}
		return listaNews;
	}

	getRelatedList(relatedListSTR: string, base_url: string): Array<Allegato> {
		const relatedList: Array<Allegato> = [];
		if (relatedListSTR) {
			relatedListSTR.split(',')?.forEach((rls) => {
				try {
					var div = document.createElement('div');
					div.innerHTML = rls.trim();
					relatedList.push({
						nome: (div.firstChild as any).text,
						url: `${base_url}/${(div.firstChild as any).getAttribute('href')}`,
					});
				} catch (error) {
					console.log('error:', error);
				}
			});
		}
		return relatedList;
	}

	parseCorrelated(correlatedStr: string, baseUrl: string): Allegato[] {
		const listRegex = /\[.*?\]/g;
		const lists = correlatedStr.match(listRegex);
		if (lists) {
			let [links, labels] = lists.map((le) => {
				const normalizedList = le.replace('[', '').replace(']', '');
				return normalizedList.split('|');
			});

			if (links.length === labels.length) {
				const attachment: Allegato[] = links.map((le, i) => {
					const splittedNode = le.split('/');
					return {
						url: `${baseUrl}${le}`,
						nome: htmlCodeToChar(labels[i]),
						id: splittedNode[splittedNode.length - 1],
					};
				});
				return attachment;
			}
		}
		return [];
	}

	getPrimoPianoHome(): Observable<NewsEventiResponse> {
		return this.httpClient.get<NewsEventiResponse>(`${PORTALE_ISTITUZIONALE_API}${PPH}`, { headers: this.getHeadersDrupal() }).pipe(
			map((response) => {
				response.rows = this.buildPathImage(response.rows);
				return response;
			})
		);
	}
}
