import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { CategoriaPoi, MarkerRequest, Poi } from 'src/app/models';
import { MACS_API, OPEN_STREET_MAPS } from 'src/app/utility';

import { ResultDTO } from '../../models/resultDTO';
import { OPEN_STREET_MAP_API } from 'src/app/utility/service.config';

const GPC = '/configure/1.0/getPoiCategories';
const SOSM = '/search?';

@Injectable({
	providedIn: 'root',
})
export class PoiService {
	constructor(private httpClient: HttpClient) { }

	getCategoriePoi(): Observable<ResultDTO<Array<CategoriaPoi>>> {
		return this.httpClient.get<ResultDTO<Array<CategoriaPoi>>>(`${MACS_API}${GPC}`);
	}

	getLocalCategoriePoi(): Observable<Array<CategoriaPoi>> {
		return this.httpClient.get<Array<CategoriaPoi>>('assets/mock/categorie-poi.json');
	}

	getMarkers(markerRequest: MarkerRequest): Observable<Poi[]> {
		let params: string = OPEN_STREET_MAPS.params_search.replace('$1', OPEN_STREET_MAPS.limit_poi.toString()).replace('$2', markerRequest.category);
		params = markerRequest.boxView ? params.replace('$4', markerRequest.boxView) : params.replace(OPEN_STREET_MAPS.viewboxParams, '');
		params = OPEN_STREET_MAPS.enableCity? params.replace('$3', OPEN_STREET_MAPS.city): params.replace(OPEN_STREET_MAPS.cityParams, '');
		return this.httpClient.get<Poi[]>(`${OPEN_STREET_MAP_API}${SOSM}${params}`);
	}
}
