import { Injectable } from "@angular/core";
import { HttpClient } from "@angular/common/http";
import { CoreRequest, Ufficio, RisultatoDTO, RisorseRequest, RisorsaRequest, Risorsa, InfoRisorsa, DownloadDocumentoRequest, DownloadDocumentoResponse, PeriodiDisponibiliRequest, PeriodiDisponibiliResponse, InitPrenotazioneRequest, Contenuto, InitPrenotazioneResponse, AnnullaPrenotazioneRequest, DisdiciPrenotazioneRequest, ConfermaPrenotazioneResponse, ListaPrenotazioni, AuditRequest, ResultDTO, ConfermaPrenotazioneTotaleRequest, Prenotazione, PrenotazioneRequest } from "../../models";
import { Observable } from "rxjs";
import { MACS_API, httpOptions } from "src/app/utility";


const GET_UFFICI: string = '/service/1.0/listaUffici';
const GET_RISORSE: string = '/service/1.0/listaRisorse';
const GET_RISORSA: string = '/service/1.0/infoRisorse';
const DOWNLOAD_DOCUMENTO: string = '/service/1.0/downloadDocumento';
const PERIODI_DISP: string = '/service/1.0/periodiDisponibili';
const INIT_PRENOTAZIONE: string = '/service/1.0/initPrenotazione';
const ANNULLA_PRENOTAZIONE: string = '/service/1.0/anullaPrenotazione';
const CONFERMA_PRENOTAZIONE: string = '/service/1.0/confermaPrenotazione';
const DISDICI_PRENOTAZIONE: string = '/service/1.0/disdiciPrenotazione';
const GET_PRENOTAZIONI: string = '/service/1.0/listaPrenotazioni';
const GET_PRENOTAZIONE: string = '/service/1.0/prenotazioneById';
const AUDIT: string = '/service/1.0/audit';

@Injectable({
  providedIn: 'root'
})
export class PrenotazioneService {


  constructor(private httpClient: HttpClient) { }

  getUffici(coreRequest: CoreRequest): Observable<RisultatoDTO<Contenuto<Array<Ufficio>>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<Array<Ufficio>>>>(`${MACS_API}${GET_UFFICI}`, JSON.stringify(coreRequest), httpOptions);
  }

  getRisorse(risorseRequest: RisorseRequest): Observable<RisultatoDTO<Contenuto<Array<Risorsa>>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<Array<Risorsa>>>>(`${MACS_API}${GET_RISORSE}`, JSON.stringify(risorseRequest), httpOptions);
  }

  getInfoRisorsa(risorsaRequest: RisorsaRequest): Observable<RisultatoDTO<Contenuto<InfoRisorsa>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<InfoRisorsa>>>(`${MACS_API}${GET_RISORSA}`, JSON.stringify(risorsaRequest), httpOptions);
  }

  downloadDocumento(downloadDocRequest: DownloadDocumentoRequest): Observable<RisultatoDTO<Contenuto<DownloadDocumentoResponse>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<DownloadDocumentoResponse>>>(`${MACS_API}${DOWNLOAD_DOCUMENTO}`, JSON.stringify(downloadDocRequest), httpOptions);
  }

  getPeriodiDisponibili(pdRequest: PeriodiDisponibiliRequest): Observable<RisultatoDTO<Contenuto<PeriodiDisponibiliResponse>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<PeriodiDisponibiliResponse>>>(`${MACS_API}${PERIODI_DISP}`, JSON.stringify(pdRequest), httpOptions);
  }

  initPrenotazione(ipRequest: InitPrenotazioneRequest): Observable<RisultatoDTO<Contenuto<InitPrenotazioneResponse>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<InitPrenotazioneResponse>>>(`${MACS_API}${INIT_PRENOTAZIONE}`, JSON.stringify(ipRequest), httpOptions);
  }

  annullaPrenotazione(apRequest: AnnullaPrenotazioneRequest): Observable<RisultatoDTO<Contenuto<String>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<String>>>(`${MACS_API}${ANNULLA_PRENOTAZIONE}`, JSON.stringify(apRequest), httpOptions);
  }

  disdiciPrenotazione(dpRequest: DisdiciPrenotazioneRequest): Observable<RisultatoDTO<Contenuto<String>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<String>>>(`${MACS_API}${DISDICI_PRENOTAZIONE}`, JSON.stringify(dpRequest), httpOptions);
  }

  confermaPrenotazione(cptRequest: ConfermaPrenotazioneTotaleRequest): Observable<RisultatoDTO<Contenuto<ConfermaPrenotazioneResponse>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<ConfermaPrenotazioneResponse>>>(`${MACS_API}${CONFERMA_PRENOTAZIONE}`, JSON.stringify(cptRequest), httpOptions);
  }

  getPrenotazioni(coreRequest: CoreRequest): Observable<RisultatoDTO<Contenuto<ListaPrenotazioni>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<ListaPrenotazioni>>>(`${MACS_API}${GET_PRENOTAZIONI}`, JSON.stringify(coreRequest), httpOptions);
  }

  getPrenotazioneByID(prenotazioneRequest: PrenotazioneRequest): Observable<RisultatoDTO<Contenuto<ListaPrenotazioni>>> {
    return this.httpClient.post<RisultatoDTO<Contenuto<ListaPrenotazioni>>>(`${MACS_API}${GET_PRENOTAZIONE}`, JSON.stringify(prenotazioneRequest), httpOptions);
  }

  audit(auditRequest: AuditRequest): Observable<ResultDTO<string>> {
    return this.httpClient.post<ResultDTO<string>>(`${MACS_API}${AUDIT}`, auditRequest, httpOptions);
  }
}
