import { Injectable } from '@angular/core';
import { Device } from '@ionic-native/device/ngx';
import { Platform } from '@ionic/angular';
import { FCM } from 'cordova-plugin-fcm-with-dependecy-updated/ionic/ngx';
import { BehaviorSubject } from 'rxjs';
import { FirebaseRequest } from 'src/app/models';
import { FIREBASE } from 'src/app/utility/global-constant';

import { StorageService } from '../';
import { FirebaseService } from '../firebase/firebase.service';
import { UtenzaService } from '../utenza/utenza.service';

@Injectable({
	providedIn: 'root',
})
export class UtilityService {
	updateSlideHome$: BehaviorSubject<boolean>;
	updateSlidePoi$: BehaviorSubject<boolean>;
	updateProfilazione$: BehaviorSubject<boolean>;

	constructor(
		private storageService: StorageService,
		private firebaseServices: FirebaseService,
		private utenzaService: UtenzaService,
		private platform: Platform,
		private fcm: FCM,
		public device: Device
	) {
		this.setUtility();
	}

	setUtility() {
		this.updateSlideHome$ = new BehaviorSubject<boolean>(false);
		this.updateSlidePoi$ = new BehaviorSubject<boolean>(false);
		this.updateProfilazione$ = new BehaviorSubject<boolean>(false);
	}

	sendTokenFirebase() {
		if (!this.platform.is('cordova')) {
			this.storageService.getUserInfo().then((infoUser) => {
				if (infoUser) {
					let firebaseRequest: FirebaseRequest = {
						codicefiscale: infoUser.identifier,
						spidCode: infoUser.identifier,
						tokenFirebase: FIREBASE.tokenMock,
						uuid: this.device.uuid,
						piattaforma: this.device.platform,
					};
					this.firebaseServices.sendTokenFirebase(firebaseRequest).subscribe(
						(result) => {
							if (result.success) {
								//TODO
								this.getDettaglioUtenza();
							}
						},
						() => {}
					);
				}
			});
		} else {
			this.fcm
				.getToken()
				.then((tokenfirebase) => {
					this.storageService.getUserInfo().then((infoUser) => {
						if (infoUser) {
							let firebaseRequest: FirebaseRequest = {
								codicefiscale: infoUser.identifier,
								spidCode: infoUser.identifier,
								tokenFirebase: tokenfirebase,
								uuid: this.device.uuid,
								piattaforma: this.device.platform,
							};
							this.firebaseServices.sendTokenFirebase(firebaseRequest).subscribe(
								(result) => {
									if (result.success) {
										this.getDettaglioUtenza();
									}
								},
								() => {}
							);
						}
					});
				})
				.catch((err) => {
					console.log('[UtilityService - sendTokenFirebase - FCM ERROR]: ', err);
				});
		}
	}

	getDettaglioUtenza() {
		this.storageService.getUserInfo().then((infoUser) => {
			if (infoUser) {
				this.utenzaService.getDettaglioUtenza(infoUser.identifier).subscribe((result) => {
					if (result.code >= 200 && result.code < 300) {
						if (result.success && result.data) {
							this.storageService.storeDettaglioUtenza(result?.data);
							this.updateProfilazione$.next(true);
						} else {
						}
					} else {
					}
				});
			}
		});
	}
}
