export const ASCII_CHAR_REGEX = /[u]{1}[0-9A-F]{4}/gi;

export const ASCII_CHARACTER = {
    u0022: '"',
    u003F: '?',
    u0021: '!',
    u0027: "'",
    u00E0: 'à',
    u00E8: 'è',
    u00EC: 'ì',
    u00ec: 'ì',
    u00F2: 'ò',
    u00F9: 'ù',
    u2019: "'",
/*     '*': '"', */
    'u0026nbsp;': ' ',
};

export const HTML_CODE_REGEX = /&#\d+;/g;

export const HTML_LITERAL_CODES = {
    '&quot;': '"',
    '&amp;': '&',
    '&lt;': '<',
    '&gt;': '>',
    '&nbsp;': ' ',
    '&iexcl;': '¡',
    '&cent;': '¢',
    '&pound;': '£',
    '&curren;': '¤',
    '&yen;': '¥',
    '&brvbar;': '¦',
    '&sect;': '§',
    '&uml;': '¨',
    '&copy;': '©',
    '&ordf;': 'ª',
    '&laquo;': '«',
    '&not;': '¬',
    '&reg;': '®',
    '&macr;': '¯',
    '&deg;': '°',
    '&plusmn;': '±',
    '&sup2;': '²',
    '&sup3;': '³',
    '&acute;': '´',
    '&micro;': 'µ',
    '&para;': '¶',
    '&middot;': '·',
    '&cedil;': '¸',
    '&sup1;': '¹',
    '&ordm;': 'º',
    '&raquo;': '»',
    '&frac14;': '¼',
    '&frac12;': '½',
    '&frac34;': '¾',
    '&iquest;': '¿',
    '&Agrave;': 'À',
    '&Aacute;': 'Á',
    '&Acirc;': 'Â',
    '&Atilde;': 'Ã',
    '&Auml;': 'Ä',
    '&Aring;': 'Å',
    '&AElig;': 'Æ',
    '&Ccedil;': 'Ç',
    '&Egrave;': 'È',
    '&Eacute;': 'É',
    '&Ecirc;': 'Ê',
    '&Euml;': 'Ë',
    '&Igrave;': 'Ì',
    '&Iacute;': 'Í',
    '&Icirc;': 'Î',
    '&Iuml;': 'Ï',
    '&ETH;': 'Ð',
    '&Ntilde;': 'Ñ',
    '&Ograve;': 'Ò',
    '&Oacute;': 'Ó',
    '&Ocirc;': 'Ô',
    '&Otilde;': 'Õ',
    '&Ouml;': 'Ö',
    '&times;': '×',
    '&Oslash;': 'Ø',
    '&Ugrave;': 'Ù',
    '&Uacute;': 'Ú',
    '&Ucirc;': 'Û',
    '&Uuml;': 'Ü',
    '&Yacute;': 'Ý',
    '&THORN;': 'Þ',
    '&szlig;': 'ß',
    '&agrave;': 'à',
    '&aacute;': 'á',
    '&acirc;': 'â',
    '&atilde;': 'ã',
    '&auml;': 'ä',
    '&aring;': 'å',
    '&aelig;': 'æ',
    '&ccedil;': 'ç',
    '&egrave;': 'è',
    '&eacute;': 'é',
    '&ecirc;': 'ê',
    '&euml;': 'ë',
    '&igrave;': 'ì',
    '&iacute;': 'í',
    '&icirc;': 'î',
    '&iuml;': 'ï',
    '&eth;': 'ð',
    '&ntilde;': 'ñ',
    '&ograve;': 'ò',
    '&oacute;': 'ó',
    '&ocirc;': 'ô',
    '&otilde;': 'õ',
    '&ouml;': 'ö',
    '&divide;': '÷',
    '&oslash;': 'ø',
    '&ugrave;': 'ù',
    '&uacute;': 'ú',
    '&ucirc;': 'û',
    '&uuml;': 'ü',
    '&yacute;': 'ý',
    '&thorn;': 'þ',
    '&yuml;': 'ÿ',
    '&euro;': '€',
};

export function htmlCodeToChar(text: string): string {
    text = formatLiteralHTMLCode(text);
    const matches = text.match(HTML_CODE_REGEX);
    if (matches) {
        matches.forEach((m) => {
            const rgx = new RegExp(m, 'gi');
            const htmlCode = m.match(/\d+/g);
            if (htmlCode) {
                text = text.replace(rgx, String.fromCharCode(Number(htmlCode[0])));
            }
        });
    }
    return text;
}

function formatLiteralHTMLCode(text: any) {
    if (text && text.match(HTML_LITERAL_CODES)) {
        Object.entries(HTML_LITERAL_CODES)?.forEach((asciiArr) => {
            const incaseSensitiveReg = new RegExp(asciiArr[0], 'g');
            if (text?.match(incaseSensitiveReg)) {
                text = text.replace(incaseSensitiveReg, asciiArr[1]);
            }
        });
    }
    return text;
}

export function normalizeImgTag(text: string, baseUrl: string) {
    if (text) {
        const imgRegex = new RegExp('(<img).*?(/>)', 'g');
        const srcRegex = new RegExp('(src=").*?(")', 'g');

        let imgMatches = text.match(imgRegex);
        if (imgMatches) {
            imgMatches.forEach((im) => {
                let srcMatches = im.match(srcRegex);
                if (srcMatches && !srcMatches[0].includes('test-')) {
                    let url = srcMatches[0];
                    url = url.replace('src="', '').replace('"', '');
                    const srcNew = im.replace(srcMatches[0], `src="${baseUrl}/${url}"`);
                    text = text.replace(im, srcNew);
                }
            });
        }
    }
    return text;
}
