package it.itsvil.notificatore.backend.service;

import java.time.LocalDateTime;

import org.apache.commons.lang3.StringUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import it.itsvil.notificatore.backend.dao.NotificatoreConfigurationDao;
import it.itsvil.notificatore.backend.dto.EmailAccountDTO;
import it.itsvil.notificatore.backend.dto.EmailBodyDTO;
import it.itsvil.notificatore.backend.dto.EmailConfigurationRequestDTO;
import it.itsvil.notificatore.backend.dto.EmailTipologyDTO;
import it.itsvil.notificatore.backend.dto.InsertEmailBodyRequestDTO;
import it.itsvil.notificatore.backend.dto.InsertNotificationConfRequestDTO;
import it.itsvil.notificatore.backend.dto.InsertUserNotificationRequestDTO;
import it.itsvil.notificatore.backend.dto.KeysDTO;
import it.itsvil.notificatore.backend.dto.NotificationConfDTO;
import it.itsvil.notificatore.backend.dto.UpdateUserNotificationRequestDTO;
import it.itsvil.notificatore.backend.dto.UserNotificationDTO;
import it.itsvil.notificatore.backend.utils.NotificatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "NotificatoreConfigurationServiceImpl")
public class NotificatoreConfigurationServiceImpl implements NotificatoreConfigurationService{
	
	@Autowired
	NotificatoreConfigurationDao dao;
	
	@Autowired
	private PlatformTransactionManager transactionManager;

	@Override
	public void insertKeys(KeysDTO request) {
		dao.insertKeys(request);
	}

	@Override
	public void insertUserNotification(InsertUserNotificationRequestDTO request) {
		UserNotificationDTO notification = new UserNotificationDTO();
		notification.setN_id_app(request.getN_id_app());
		notification.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
		notification.setD_data_ins(LocalDateTime.now());
		dao.insertUserNotification(notification);
	}

	@Override
	public void updateUserNotification(UpdateUserNotificationRequestDTO request) {
		UserNotificationDTO notification = new UserNotificationDTO();
		notification.setN_id_app(request.getN_id_app());
		notification.setB_attivo(request.getB_attivo());
		dao.updateUserNotification(notification);
		
	}

	@Override
	public void insertNotificationConf(InsertNotificationConfRequestDTO request) {
		NotificationConfDTO conf = new NotificationConfDTO();
		conf.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
		conf.setN_id_app(request.getN_id_app());
		conf.setT_key(request.getT_key());
		conf.setT_value(request.getT_value());
		dao.insertNotificationConf(conf);
	}

	@Override
	public void updateNotificationConf(NotificationConfDTO request) {
		dao.updateNotificationConf(request);
	}

	@Override
	public void insertEmailBody(InsertEmailBodyRequestDTO request) {
		EmailBodyDTO email = new EmailBodyDTO();
		byte[] decodeBody = Base64.decodeBase64(request.getT_encoded_Body());
		email.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
		email.setN_id_app(request.getN_id_app());
		email.setT_corpo(decodeBody);
		email.setT_encoded_Body(request.getT_encoded_Body());
		email.setT_tipo_email(request.getT_tipo_email());
		dao.insertEmailBody(email);
	}

	@Override
	public void updateEmailBody(EmailBodyDTO request) {
		if (request.getT_encoded_Body()!=null) {
			byte[] decodeBody = Base64.decodeBase64(request.getT_encoded_Body());
			request.setT_corpo(decodeBody);
		}
		dao.updateEmailBody(request);
	}
	
	@Override
	public int insertEmailConfiguration(EmailConfigurationRequestDTO request, String n_id_app) {

		EmailAccountDTO email_accountDTO = request.getEmailAccountDTO();
		EmailTipologyDTO email_tipologyDTO = request.getEmailTipologyDTO();

		TransactionStatus txStatus = transactionManager.getTransaction(new DefaultTransactionDefinition());

		try {

			if (null != request && StringUtils.isNotBlank(email_accountDTO.getEmail_account()) && StringUtils.isNotBlank(email_accountDTO.getEmail_password()) && StringUtils.isNotBlank(email_accountDTO.getEmail_port()) && StringUtils.isNotBlank(email_accountDTO.getEmail_host_smtp())) {

				NotificationConfDTO notification_ConfDTO1 = new NotificationConfDTO();
				NotificationConfDTO notification_ConfDTO2 = new NotificationConfDTO();
				NotificationConfDTO notification_ConfDTO3 = new NotificationConfDTO();
				NotificationConfDTO notification_ConfDTO4 = new NotificationConfDTO();

				if(StringUtils.isNotBlank(email_tipologyDTO.getBody_generale())) {
					EmailBodyDTO email_BodyDTO1 = new EmailBodyDTO();
					email_BodyDTO1.setN_id_app(n_id_app);
					email_BodyDTO1.setT_tipo_email(NotificatoreEnum.BODY_GENERALE.getCodice());
					email_BodyDTO1.setT_encoded_Body(email_tipologyDTO.getBody_generale());
					byte[] decodeBody1 = Base64.decodeBase64(email_BodyDTO1.getT_encoded_Body());
					email_BodyDTO1.setT_corpo(decodeBody1);
					email_BodyDTO1.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO1);
				}

				if(StringUtils.isNotBlank(email_tipologyDTO.getBody_recupera_password())) {
					EmailBodyDTO email_BodyDTO2 = new EmailBodyDTO();
					email_BodyDTO2.setN_id_app(n_id_app);
					email_BodyDTO2.setT_tipo_email(NotificatoreEnum.BODY_RECUPERA_PASSWORD.getCodice());
					email_BodyDTO2.setT_encoded_Body(email_tipologyDTO.getBody_recupera_password());
					byte[] decodeBody2 = Base64.decodeBase64(email_BodyDTO2.getT_encoded_Body());
					email_BodyDTO2.setT_corpo(decodeBody2);
					email_BodyDTO2.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO2);
				}

				if(StringUtils.isNotBlank(email_tipologyDTO.getBody_registrazione())) {
					EmailBodyDTO email_BodyDTO3 = new EmailBodyDTO();
					email_BodyDTO3.setN_id_app(n_id_app);
					email_BodyDTO3.setT_tipo_email(NotificatoreEnum.BODY_REGISTRAZIONE.getCodice());
					email_BodyDTO3.setT_encoded_Body(email_tipologyDTO.getBody_registrazione());
					byte[] decodeBody3 = Base64.decodeBase64(email_BodyDTO3.getT_encoded_Body());
					email_BodyDTO3.setT_corpo(decodeBody3);
					email_BodyDTO3.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO3);

				}

				if(StringUtils.isNotBlank(email_tipologyDTO.getBody_info1())) {
					EmailBodyDTO email_BodyDTO4 = new EmailBodyDTO();
					email_BodyDTO4.setN_id_app(n_id_app);
					email_BodyDTO4.setT_tipo_email(NotificatoreEnum.BODY_INFO_1.getCodice());
					email_BodyDTO4.setT_encoded_Body(email_tipologyDTO.getBody_info1());
					byte[] decodeBody4 = Base64.decodeBase64(email_BodyDTO4.getT_encoded_Body());
					email_BodyDTO4.setT_corpo(decodeBody4);
					email_BodyDTO4.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO4);

				}

				if (StringUtils.isNotBlank(email_tipologyDTO.getBody_info2())) {
					EmailBodyDTO email_BodyDTO5 = new EmailBodyDTO();
					email_BodyDTO5.setN_id_app(n_id_app);
					email_BodyDTO5.setT_tipo_email(NotificatoreEnum.BODY_INFO_2.getCodice());
					email_BodyDTO5.setT_encoded_Body(email_tipologyDTO.getBody_info2());
					byte[] decodeBody5 = Base64.decodeBase64(email_BodyDTO5.getT_encoded_Body());
					email_BodyDTO5.setT_corpo(decodeBody5);
					email_BodyDTO5.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO5);

				}

				if(StringUtils.isNotBlank(email_tipologyDTO.getBody_conferma_operazione())) {
					EmailBodyDTO email_BodyDTO6 = new EmailBodyDTO();
					email_BodyDTO6.setN_id_app(n_id_app);
					email_BodyDTO6.setT_tipo_email(NotificatoreEnum.BODY_CONFERMA_OPERAZIONE.getCodice());
					email_BodyDTO6.setT_encoded_Body(email_tipologyDTO.getBody_conferma_operazione());
					byte[] decodeBody6 = Base64.decodeBase64(email_BodyDTO6.getT_encoded_Body());
					email_BodyDTO6.setT_corpo(decodeBody6);
					email_BodyDTO6.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());	
					dao.insertEmailBody(email_BodyDTO6);

				}


				notification_ConfDTO1.setN_id_app(n_id_app);
				notification_ConfDTO1.setT_key(NotificatoreEnum.EMAIL_ACCOUNT.getCodice());
				notification_ConfDTO1.setT_value(email_accountDTO.getEmail_account());
				notification_ConfDTO1.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
				dao.insertNotificationConf(notification_ConfDTO1);

				notification_ConfDTO2.setN_id_app(n_id_app);
				notification_ConfDTO2.setT_key(NotificatoreEnum.EMAIL_PASSWORD.getCodice());
				notification_ConfDTO2.setT_value(email_accountDTO.getEmail_password());
				notification_ConfDTO2.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
				dao.insertNotificationConf(notification_ConfDTO2);

				notification_ConfDTO3.setN_id_app(n_id_app);
				notification_ConfDTO3.setT_key(NotificatoreEnum.EMAIL_PORT.getCodice());
				notification_ConfDTO3.setT_value(email_accountDTO.getEmail_port());
				notification_ConfDTO3.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
				dao.insertNotificationConf(notification_ConfDTO3);

				notification_ConfDTO4.setN_id_app(n_id_app);
				notification_ConfDTO4.setT_key(NotificatoreEnum.EMAIL_HOST_SMTP.getCodice());
				notification_ConfDTO4.setT_value(email_accountDTO.getEmail_host_smtp());
				notification_ConfDTO4.setB_attivo(NotificatoreEnum.ATTIVO.getCodice());
				dao.insertNotificationConf(notification_ConfDTO4);

				transactionManager.commit(txStatus);

				return 0; 

			}
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			transactionManager.rollback(txStatus); 
			return -1;
		}

		return 1;

	}
	
}
