package it.itsvil.notificatore.rest;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.notificatore.backend.dto.EmailBodyDTO;
import it.itsvil.notificatore.backend.dto.EmailConfigurationRequestDTO;
import it.itsvil.notificatore.backend.dto.InsertEmailBodyRequestDTO;
import it.itsvil.notificatore.backend.dto.InsertNotificationConfRequestDTO;
import it.itsvil.notificatore.backend.dto.InsertUserNotificationRequestDTO;
import it.itsvil.notificatore.backend.dto.KeysDTO;
import it.itsvil.notificatore.backend.dto.NotificationConfDTO;
import it.itsvil.notificatore.backend.dto.RisultatoDTO;
import it.itsvil.notificatore.backend.dto.UpdateUserNotificationRequestDTO;
import it.itsvil.notificatore.backend.service.NotificatoreConfigurationService;
import it.itsvil.notificatore.backend.utils.NotificatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/configuration")
public class NotificatoreConfigurationServiceRest {
	
	@Autowired
	private NotificatoreConfigurationService service;
	
	public ResponseEntity<RisultatoDTO<String>> fallbackInsertKey(@RequestBody KeysDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		response.error("Il sistema al momento non è disponibile",HttpStatus.SC_EXPECTATION_FAILED);
		return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
	}
	
	@PutMapping(value = "/1.0/insertKey", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertKey(@RequestBody KeysDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getT_key())) {
				service.insertKeys(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);


	}
	
	@PutMapping(value = "/1.0/insertUserNotification", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertUserNotification(@RequestBody InsertUserNotificationRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getN_id_app())) {
				service.insertUserNotification(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateUserNotification", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateUserNotification(@RequestBody UpdateUserNotificationRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getB_attivo()) && StringUtils.isNotBlank(request.getN_id_app()) 
					&& (NotificatoreEnum.ATTIVO.getCodice().equals(request.getB_attivo()) || NotificatoreEnum.NON_ATTIVO.getCodice().equals(request.getB_attivo()))) {
				service.updateUserNotification(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);


	}
	
	@PutMapping(value = "/1.0/insertNotificationConf", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertNotificationConf(@RequestBody InsertNotificationConfRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getN_id_app()) && StringUtils.isNotBlank(request.getT_key()) && StringUtils.isNotBlank(request.getT_value())) {
				service.insertNotificationConf(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateNotificationConf", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateNotificationConf(@RequestBody NotificationConfDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getB_attivo()) && StringUtils.isNotBlank(request.getT_value()) && StringUtils.isNotBlank(request.getN_id_app()) && 
					StringUtils.isNotBlank(request.getT_key()) && (NotificatoreEnum.ATTIVO.getCodice().equals(request.getB_attivo()) || NotificatoreEnum.NON_ATTIVO.getCodice().equals(request.getB_attivo()))) {
				service.updateNotificationConf(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PutMapping(value = "/1.0/insertEmailBody", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertEmailBody(@RequestBody InsertEmailBodyRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getN_id_app()) && StringUtils.isNotBlank(request.getT_encoded_Body()) && StringUtils.isNotBlank(request.getT_tipo_email())) {
				service.insertEmailBody(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateEmailBody", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateEmailBody(@RequestBody EmailBodyDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (null != request && StringUtils.isNotBlank(request.getB_attivo()) && StringUtils.isNotBlank(request.getT_tipo_email()) && StringUtils.isNotBlank(request.getN_id_app()) && 
					StringUtils.isNotBlank(request.getT_encoded_Body()) &&  (null == request.getB_attivo() || NotificatoreEnum.ATTIVO.getCodice().equals(request.getB_attivo()) || NotificatoreEnum.NON_ATTIVO.getCodice().equals(request.getB_attivo()))) {
				service.updateEmailBody(request);
				response.setDescrizione("inserimento avvenuto con successo");
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PutMapping(value = "/1.0/insertEmailConfiguration", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertEmailConfiguration(@RequestBody EmailConfigurationRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		int esito = 1;

		try {
			if(null != request && StringUtils.isNotBlank(request.getN_id_app()) && null != request.getEmailAccountDTO() && null != request.getEmailTipologyDTO() &&
					StringUtils.isNotBlank(request.getEmailAccountDTO().getEmail_account()) && StringUtils.isNotBlank(request.getEmailAccountDTO().getEmail_host_smtp()) &&
					StringUtils.isNotBlank(request.getEmailAccountDTO().getEmail_password()) && StringUtils.isNotBlank(request.getEmailAccountDTO().getEmail_port())) {
				
				esito = service.insertEmailConfiguration(request, request.getN_id_app());

				if(esito == 0) {
					response.setDescrizione("inserimento avvenuto con successo");
					response.success(HttpStatus.SC_OK);
				}else if(esito == 1){
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}else{
					response.error("Impossibile inserire la configurazione",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else{
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch(Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}

		return ResponseEntity.ok(response);
	}
	
}
