package it.itsvil.profilatore.rest;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import it.itsvil.profilatore.backend.dto.InsertFunzionalitaAggiuntiveRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzioneFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertFunzioneRequestDTO;
import it.itsvil.profilatore.backend.dto.InsertRuoloRequestDTO;
import it.itsvil.profilatore.backend.dto.RisultatoDTO;
import it.itsvil.profilatore.backend.dto.UpdateFunzionalitaRequestDTO;
import it.itsvil.profilatore.backend.dto.UpdateFunzioneRequestDTO;
import it.itsvil.profilatore.backend.dto.UpdateRuoloRequestDTO;
import it.itsvil.profilatore.backend.service.ProfilatoreConfigurationService;
import it.itsvil.profilatore.backend.utils.ProfilatoreEnum;
import lombok.extern.log4j.Log4j2;

@Log4j2
@RestController
@RequestMapping(value= "/configuration")
public class ProfilatoreConfigurationServiceRest {

	@Autowired
	ProfilatoreConfigurationService service;
	
	@PutMapping(value = "/1.0/insertFunzione", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertFunzione(@RequestBody InsertFunzioneRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (request!= null && StringUtils.isNotBlank(request.getT_nome()) && null != request.getN_id_ruolo()) {
				
				int esito=0;
				esito=service.insertFunzione(request);
				if(esito==0) {
					response.setDescrizione("inserimento avvenuto con successo");
					response.setData("Inserimento avvenuto con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'inserimento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);

	}
	
	@PostMapping(value = "/1.0/updateFunzione" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateFunzione(@RequestBody UpdateFunzioneRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {

			if (null != request && null != request.getN_id_funzione() && null != request.getN_id_ruolo()) {
				if(StringUtils.isNotBlank(request.getB_attivo()) && (!request.getB_attivo().equals(ProfilatoreEnum.ATTIVO.getCodice()) && !request.getB_attivo().equals(ProfilatoreEnum.NON_ATTIVO.getCodice()))){
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}
				int esito = service.updateFunzione(request);
				if(esito == 0){
					response.setDescrizione("Funzione correttamente aggiornata");
					response.success(HttpStatus.SC_OK);
					response.setData("Funzione correttamente aggiornata");
				}else{
					response.error("Impossibile aggiornare la funzione",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
				response.success(HttpStatus.SC_OK);
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}

		return ResponseEntity.ok(response);

	}

	@PutMapping(value = "/1.0/insertFunzionalita", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertFunzionalita(@RequestBody InsertFunzionalitaRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			if (request!= null && StringUtils.isNotBlank(request.getT_nome_funzionalita()) && StringUtils.isNotBlank(request.getT_livello_sicurezza())) {
				
				int esito=0;
				esito=service.insertFunzionalita(request);
				if(esito==0) {
					response.setDescrizione("Inserimento avvenuto con successo");
					response.setData("Inserimento avvenuto con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'inserimento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);

	}


	@PostMapping(value = "/1.0/updateFunzionalita" , consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateFunzionalita(@RequestBody UpdateFunzionalitaRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {

			if (null!=request && null!=request.getN_id_funzionalita()) {
				if(StringUtils.isNotBlank(request.getB_attivo()) && (!request.getB_attivo().equals(ProfilatoreEnum.ATTIVO.getCodice()) && !request.getB_attivo().equals(ProfilatoreEnum.NON_ATTIVO.getCodice()))){
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}
				int result=service.setFunzionalita(request);
				if(result ==0) {
					response.setDescrizione("Funzionalita correttamente aggiornata");
					response.setData("Funzionalità correttamente aggiornata");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'aggiornamento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}

		return ResponseEntity.ok(response);

	}
	
	@PutMapping(value = "/1.0/insertRuolo", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertRuolo(@RequestBody InsertRuoloRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {
			if (null!=request && StringUtils.isNotBlank(request.getN_id_app()) && StringUtils.isNotBlank(request.getT_nome())) {
				
				int result=service.insertRuolo(request);
				if (result == 0) {
					response.setDescrizione("Inserimento avvenuto con successo");
					response.setData("Inserimento avvenuto con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'inserimento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
			
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PostMapping(value = "/1.0/updateRuolo", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateRuolo(@RequestBody UpdateRuoloRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {
			if (request != null && null != request.getN_id_ruolo()) {
				if(StringUtils.isNotBlank(request.getB_attivo()) && (!request.getB_attivo().equals(ProfilatoreEnum.ATTIVO.getCodice()) && !request.getB_attivo().equals(ProfilatoreEnum.NON_ATTIVO.getCodice()))){
					response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
					return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
				}
				int result=service.setRuolo(request);
				if (result == 0) {
					response.setData("Aggiornamento effettuato con successo");
					response.setDescrizione("Aggiornamento effettuato con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'aggiornamento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
			
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PutMapping(value = "/1.0/insertFunzioneFunzionalita", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertFunzioneFunzionalita(@RequestBody InsertFunzioneFunzionalitaRequestDTO request){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {
			if (request!= null && null != request.getN_id_funzione() && null != request.getN_id_funzionalita()) {
				
				int result=service.insertFunzioneFunzionalita(request);
				if (result == 0) {
					response.setData("Inserimento effettuato con successo");
					response.setDescrizione("Inserimento effettuato con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'inserimento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
			
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@DeleteMapping(value = "/1.0/deleteFunzioneFunzionalita", produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> deleteFunzioneFunzionalita(@RequestParam("n_id_funzione") Long n_id_funzione, @RequestParam("n_id_funzionalita") Long n_id_funzionalita){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		
		try {
			if (null != n_id_funzione && null != n_id_funzionalita) {
				
				int result=service.deleteFunzioneFunzionalita(n_id_funzione,n_id_funzionalita);
				if (result == 0) {
					response.setData("Cancellazione avvenuta con successo");
					response.setDescrizione("Cancellazione avvenuta con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare la cancellazione",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
			
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);
	}
	
	@PutMapping(value = "/1.0/insertFunzionalitaAggiuntiva", consumes = {MediaType.APPLICATION_JSON_VALUE}, produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> insertFunzionalitaAggiuntive(@RequestBody InsertFunzionalitaAggiuntiveRequestDTO request){

		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {
			
			if (request != null && null != request.getN_id_funzionalita() && null != request.getN_id_funzione() && null != request.getN_id_utente()) {
				int esito;
				esito=service.insertFunzionalitaAggiuntive(request);
				if(esito==0) {
					response.setDescrizione("Inserimento effettuato con successo");
					response.setData("Inserimento effettuato con successo");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'inserimento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}
		return ResponseEntity.ok(response);

	}
	
	@DeleteMapping(value = "/1.0/deleteFunzionalitaAggiuntiva", produces = {MediaType.APPLICATION_JSON_VALUE})
	@CrossOrigin
	public ResponseEntity<RisultatoDTO<String>> updateFunzionalitaAggiuntive(@RequestParam("n_id_funzionalita") Long n_id_funzionalita, @RequestParam("n_id_utente") Long n_id_utente, @RequestParam("n_id_funzione") Long n_id_funzione){
		RisultatoDTO<String> response = new RisultatoDTO<String>();
		try {

			if (null != n_id_funzionalita && null != n_id_funzione && null != n_id_utente) {
				int result=service.deleteFunzionalitaAggiuntive(n_id_funzionalita,n_id_utente,n_id_funzione);
				if(result ==0) {
					response.setDescrizione("Funzionalita aggiuntiva correttamente cancellata");
					response.setData("Funzionalita aggiuntiva correttamente cancellata");
					response.success(HttpStatus.SC_OK);
				}else {
					response.error("Impossibile effettuare l'aggiornamento",HttpStatus.SC_EXPECTATION_FAILED);
					return ResponseEntity.status(HttpStatus.SC_EXPECTATION_FAILED).body(response);
				}
			}else {
				response.error("Bad Request",HttpStatus.SC_BAD_REQUEST);
				return ResponseEntity.status(HttpStatus.SC_BAD_REQUEST).body(response);
			}
		}catch (Exception e) {
			return ResponseEntity.status(HttpStatus.SC_INTERNAL_SERVER_ERROR).body(response);
		}

		return ResponseEntity.ok(response);

	}
	
	
}
